begin;

-- reports constant data in stored procs
create or replace function ins_size_ranges() returns void as '
declare
    exponent int := 0;
    magnitude constant int := 10;
    multiplier int;
    low bigint;
    high bigint;
begin   
    while exponent <= magnitude loop
        multiplier := 1;
        while multiplier <= 9 loop
            low := (10 ^ exponent) * multiplier;
            high := ((10 ^ exponent) * (multiplier + 1)) - 1;
            insert into prd_msg_size_range (lo, hi) values (low, high);
            multiplier := multiplier + 1;
        end loop;
        exponent := exponent + 1;
    end loop;
    return;
end' language plpgsql;

create or replace function ins_probability_ranges() returns void as '
declare
    granularity int := 2;
    bin_count int;
    bin int := 0;
    low int;
    high int;
begin   
    bin_count = 100 / granularity;
    while bin < bin_count loop
        low := bin * granularity;
        high := (bin + 1) * granularity;
        insert into prd_probability_range (lo, hi) values (low, high);
        bin := bin + 1;
    end loop;
    return;
end' language plpgsql;

delete from prd_msg_size_range;
select ins_size_ranges();
delete from prd_probability_range;
select ins_probability_ranges();

-- generalization of the quarantine action request queue
create table qrntn_action_request (
    submission_time timestamp with time zone default current_timestamp,
    m_global_id character varying(255) not null,
    location_name character varying(255) not null,
    action smallint default 0 not null,
    local character varying(255) not null,
    domain character varying(255)
);

insert into qrntn_action_request (submission_time, m_global_id, local, domain)
    select submission_time, m_global_id, local, domain
        from msg_release_request;

alter table only qrntn_action_request
    add constraint unq_qrntn_action_request unique (
        m_global_id, "action", "local", "domain"
    );

drop table msg_release_request;

create table prd_policy_mark (
    id serial not null,
    name character varying(64) not null
);

-- new policy mark count reporting tables
alter table only prd_policy_mark
    add constraint unq_prd_policy_mark unique (name);

alter table only prd_policy_mark
    add constraint pk_prd_policy_mark primary key (id);

create table prd_policy_mark_count (
    period_id integer not null,
    pmx_host_id integer not null,
    policy_mark_id integer,
    tally integer
);

-- unnecessary index on quarantine_rule_hit table
drop index quarantine_rule_hit_quarantine_rule_id_index;

COMMENT ON SCHEMA public IS 'PMX 5.003';

commit;
