begin;

create view prd_msgs_processed_count_per_reason_view as
    select  Period.period_id,
            prd_quarantine_reason.id as quarantine_reason_id,
            MsgCount.pmx_host_id,       -- IS NULL iff "tally" == 0
            coalesce(tally,0)        as tally
    from    prd_quarantine_reason
    cross join (select distinct period_id 
                from prd_msgs_processed_count
                where quarantine_reason_id is not null) as Period
    left join prd_msgs_processed_count                  as MsgCount
    on      MsgCount.period_id = Period.period_id
    and     MsgCount.quarantine_reason_id = prd_quarantine_reason.id
;

insert into prd_quarantine_reason (name)
    select 'spam' AS name where not exists
    (select name from prd_quarantine_reason where name='spam');

insert into prd_quarantine_reason (name)
    select 'virus' AS name where not exists
    (select name from prd_quarantine_reason where name='virus');

insert into prd_quarantine_reason (name)
    select 'other' AS name where not exists
    (select name from prd_quarantine_reason where name='other');

-- generalization of the quarantine action request queue
-- we happen to need this section for some databases and not need it for others;
-- the SQL processing utility needs to know how to handle errors on commands
-- in this section
create table qrntn_action_request (
    submission_time timestamp with time zone default current_timestamp,
    m_global_id character varying(255) not null,
    location_name character varying(255) not null,
    action smallint default 0 not null,
    local character varying(255) not null,
    domain character varying(255)
);

insert into qrntn_action_request (submission_time, m_global_id,
                                  location_name, local, domain)
    select submission_time, m_global_id, location_name, local, domain
        from msg_release_request;

alter table only qrntn_action_request
    add constraint unq_qrntn_action_request unique (
        m_global_id, "action", "local", "domain"
    );

drop table msg_release_request;

COMMENT ON SCHEMA public IS 'PMX 5.004';

commit;

