begin;

-- additional column for the qrntn_action_request table
alter table qrntn_action_request add attempt_count integer;
update qrntn_action_request set attempt_count = 0;
alter table qrntn_action_request alter attempt_count set default 0;

-- new versions of the views
drop view prd_msgs_processed_count_per_reason_view;
create view prd_msgs_processed_count_per_reason_view
as
select  prd_period.id as period_id,
        prd_quarantine_reason.id as quarantine_reason_id,
        prd_pmx_host.id as pmx_host_id,
        coalesce(prd_msgs_processed_count.tally,0)        as tally,
        prd_period.duration,
        prd_period.period_start,
        prd_period.period_end
from    (prd_period cross join prd_pmx_host cross join prd_quarantine_reason)
        left join prd_msgs_processed_count
        on prd_period.id = prd_msgs_processed_count.period_id
        and prd_quarantine_reason.id = prd_msgs_processed_count.quarantine_reason_id
	and prd_pmx_host.id = prd_msgs_processed_count.pmx_host_id;

drop view prd_pmx_runtime_fact_quarantine_view;
create view     prd_pmx_runtime_fact_quarantine_view as
select          prd_period.id as period_id,
                prd_period.period_start,
                prd_period.period_end,
                prd_period.duration,
                prd_runtime_fact_type.id as runtime_fact_type_id,
                prd_pmx_host.id as pmx_host_id,
                coalesce(value_absolute, 0) as value_absolute
from            ((prd_period join prd_runtime_fact_type on exists (
                        select 'x' from prd_pmx_runtime_fact
                        where prd_pmx_runtime_fact.period_id = prd_period.id
                        and prd_runtime_fact_type.name in (
                                'total_quarantined_local',
                                'quarantine_kbytes_local',
                                'new_quarantined_local'
                        )
                )) cross join prd_pmx_host)
                left join prd_pmx_runtime_fact prd_pmx_runtime_fact_2
                on prd_period.id = prd_pmx_runtime_fact_2.period_id
                and prd_pmx_runtime_fact_2.runtime_fact_type_id = prd_runtime_fact_type.id
                and prd_pmx_runtime_fact_2.runtime_fact_type_id in (
                        select id from prd_runtime_fact_type prd_runtime_fact_type_2
                        where prd_runtime_fact_type_2.name in (
                                'total_quarantined_local',
                                'quarantine_kbytes_local',
                                'new_quarantined_local'
                        )
                and prd_pmx_host.id = prd_pmx_runtime_fact_2.pmx_host_id
                )
;

-- missing index
create index prd_msgs_processed_count_spam_virus_rule_id_index
on prd_msgs_processed_count (spam_virus_rule_id);

-- part of the upgrade to PGSQL 7.4.8
UPDATE pg_proc SET proargtypes[3] = 'internal'::regtype
WHERE pronamespace = 11 AND pronargs = 5
      AND proargtypes[2] = 'cstring'::regtype;

COMMENT ON SCHEMA public IS 'PMX 5.007';

commit;
