begin;

-- drop old view definition
drop view prd_policy_action_count_view;

-- change view definition
create view prd_policy_action_count_view
as
select     prd_period.id as period_id,
           prd_pmx_host.id as pmx_host_id,
           prd_policy_action.id as policy_action_id,
           prd_period.period_start,
           prd_period.period_end,
           prd_period.duration,
           coalesce(GROUPED_ACTION_COUNT.tally,0) as tally
from       prd_period
cross join prd_policy_action
cross join prd_pmx_host
left join  (
            select  action_count.period_id,
                    action_count.pmx_host_id,
                    action_count.policy_action_name,
                    sum(action_count.tally) as tally
            from (
                  select period_id,
                         pmx_host_id,
                         case prd_policy_action.name
                              when 'ok' then 'ok' else 'reject'
                         end
                         as policy_action_name,
                         tally
                  from   prd_msgs_processed_count
                  join   prd_policy_action
                      on prd_policy_action.id = prd_msgs_processed_count.policy_action_id
                 ) as ACTION_COUNT
            group by ACTION_COUNT.period_id, ACTION_COUNT.pmx_host_id, ACTION_COUNT.policy_action_name
           ) as GROUPED_ACTION_COUNT
    on     GROUPED_ACTION_COUNT.period_id = prd_period.id
     and   GROUPED_ACTION_COUNT.policy_action_name = prd_policy_action.name
     and   GROUPED_ACTION_COUNT.pmx_host_id = prd_pmx_host.id
where      prd_policy_action.name in ('ok', 'reject')
;

COMMENT ON SCHEMA public IS 'PMX 5.101';

commit;
