begin;

-- Add pmx_group table
create table pmx_group (
    group_name text not null,
    long_name  text not null,
    constraint pk_pmx_group primary key (group_name),
    constraint chk_pmx_group_group_name check (group_name ~ '^[a-z0-9][a-z0-9_-]+$')
);

-- add pmx_user table
create table pmx_user (
    user_name  text not null,
    passphrase text not null,
    email      text not null,
    fullname   text,
    locale     text,
    constraint pk_pmx_user primary key (user_name),
    constraint chk_pmx_user_user_name check (user_name ~ '^[a-z][a-z0-9_.-]+$')
);

-- add group_admin table
create table group_admin (
    group_admin_id serial not null,
    group_name text not null,
    user_name  text not null,
    constraint pk_group_admin primary key (group_admin_id),
    constraint fk_group_admin_group_name
        foreign key (group_name) references pmx_group (group_name)
        on delete cascade,
    constraint fk_group_admin_user_name
        foreign key (user_name) references pmx_user (user_name)
        on delete cascade,
    constraint unq_group_admin_group_name_user_name
        unique (group_name, user_name)
);

create index group_admin_group_name_index on group_admin (group_name);
create index group_admin_user_name_index on group_admin (user_name);

-- add permission table
create table permission (
    permission_name text not null,
    permission_bit integer unique not null,
    constraint pk_permission primary key (permission_name)
);

-- only allow data to be inserted on derived tables
create rule permission_for_insert as on insert to permission do nothing;

-- add binary_permission table
create table binary_permission () inherits (permission);

-- define binary permissions
insert into binary_permission values ('off', 0);
insert into binary_permission values ('on', 1);

-- add read_write_permission table
create table read_write_permission () inherits (permission);

-- define read/write permissions
insert into read_write_permission values ('read', 2);
insert into read_write_permission values ('write', 4);

-- add permission_item table
create table permission_item (
    permission_item text not null,
    default_permission_bits integer not null default 0,
    constraint pk_permission_item primary key (permission_item)
);

-- only allow data to be inserted on derived tables
create rule permission_item_for_insert as on insert to permission_item do nothing;

-- add binary_permission_item table
create table binary_permission_item (
    default_permission_bits integer not null default 1
) inherits (permission_item);

-- on/off permission item definitions
insert into binary_permission_item (permission_item) values ('help');
insert into binary_permission_item (permission_item) values ('configuration');
insert into binary_permission_item (permission_item) values ('configuration.policydescription');
insert into binary_permission_item (permission_item) values ('quarantine');
insert into binary_permission_item (permission_item) values ('quarantine.actions');
insert into binary_permission_item (permission_item) values ('quarantine.actions.approve');
insert into binary_permission_item (permission_item) values ('quarantine.actions.delete');
insert into binary_permission_item (permission_item) values ('quarantine.actions.report');
insert into binary_permission_item (permission_item) values ('quarantine.actions.forward');
insert into binary_permission_item (permission_item) values ('quarantine.actions.save');
insert into binary_permission_item (permission_item) values ('quarantine.preview');
insert into binary_permission_item (permission_item) values ('quarantine.preview.content');
insert into binary_permission_item (permission_item) values ('quarantine.preview.source');
insert into binary_permission_item (permission_item) values ('quarantine.preview.info');
insert into binary_permission_item (permission_item) values ('quarantine.preview.attachments');
insert into binary_permission_item (permission_item) values ('quarantine.preview.attachments.download');
insert into binary_permission_item (permission_item) values ('quarantine.preview.status');
insert into binary_permission_item (permission_item) values ('quarantine.reason.spam');
insert into binary_permission_item (permission_item) values ('quarantine.reason.virus');
insert into binary_permission_item (permission_item) values ('quarantine.reason.suspect');
insert into binary_permission_item (permission_item) values ('quarantine.reason.blacklisted');
insert into binary_permission_item (permission_item) values ('reports');
insert into binary_permission_item (permission_item) values ('reports.messagecategorizations');
insert into binary_permission_item (permission_item) values ('reports.spamviruscount');
insert into binary_permission_item (permission_item) values ('reports.ipblockedmessages');
insert into binary_permission_item (permission_item) values ('reports.rejectedmtaconnections');
insert into binary_permission_item (permission_item) values ('reports.spamrangevolumes');
insert into binary_permission_item (permission_item) values ('reports.topvirustypes');
insert into binary_permission_item (permission_item) values ('reports.policymarkhits');
insert into binary_permission_item (permission_item) values ('reports.toprelays');
insert into binary_permission_item (permission_item) values ('reports.topspamrelays');
insert into binary_permission_item (permission_item) values ('reports.topvirusrelays');
insert into binary_permission_item (permission_item) values ('reports.topotherrelays');
insert into binary_permission_item (permission_item) values ('reports.topspamsenders');
insert into binary_permission_item (permission_item) values ('reports.numberofreleases');
insert into binary_permission_item (permission_item) values ('reports.topreleasers');
insert into binary_permission_item (permission_item) values ('reports.topspamrecipients');
insert into binary_permission_item (permission_item) values ('reports.messagecategorizations.export');
insert into binary_permission_item (permission_item) values ('reports.spamviruscount.export');
insert into binary_permission_item (permission_item) values ('reports.ipblockedmessages.export');
insert into binary_permission_item (permission_item) values ('reports.rejectedmtaconnections.export');
insert into binary_permission_item (permission_item) values ('reports.spamrangevolumes.export');
insert into binary_permission_item (permission_item) values ('reports.topvirustypes.export');
insert into binary_permission_item (permission_item) values ('reports.policymarkhits.export');
insert into binary_permission_item (permission_item) values ('reports.toprelays.export');
insert into binary_permission_item (permission_item) values ('reports.topspamrelays.export');
insert into binary_permission_item (permission_item) values ('reports.topvirusrelays.export');
insert into binary_permission_item (permission_item) values ('reports.topotherrelays.export');
insert into binary_permission_item (permission_item) values ('reports.topspamsenders.export');
insert into binary_permission_item (permission_item) values ('reports.numberofreleases.export');
insert into binary_permission_item (permission_item) values ('reports.topreleasers.export');
insert into binary_permission_item (permission_item) values ('reports.topspamrecipients.export');


-- add read_write_permission_item table
create table read_write_permission_item (
    default_permission_bits integer not null default 6
) inherits (permission_item);

-- read/write permission item definitions
insert into read_write_permission_item (permission_item) values ('configuration.policysettings');
insert into read_write_permission_item (permission_item) values ('configuration.lists.allowed-senders-per-group');
insert into read_write_permission_item (permission_item) values ('configuration.lists.allowed-relays-per-group');
insert into read_write_permission_item (permission_item) values ('configuration.lists.blocked-senders-per-group');
insert into read_write_permission_item (permission_item) values ('configuration.lists.blocked-relays-per-group');
insert into read_write_permission_item (permission_item) values ('configuration.eu_lists.whitelisted-senders-per-user');
insert into read_write_permission_item (permission_item) values ('configuration.eu_lists.blacklisted-senders-per-user');

-- add group_admin_permission table
create table group_admin_permission (
    group_admin_permission_id serial not null,
    group_admin_id integer not null,
    permission_item text not null,
    permission_bits integer not null default 0,
    constraint pk_group_admin_permission primary key (group_admin_permission_id),
    constraint fk_group_admin_permission_group_admin_id
        foreign key (group_admin_id) references group_admin (group_admin_id)
        on delete cascade,
    constraint fk_group_admin_permission_permission_item
        foreign key (permission_item) references permission_item (permission_item)
        on delete cascade,
    constraint unq_group_admin_permission_group_admin_id_permission_item unique (group_admin_id, permission_item)
);

-- only allow data to be inserted on derived tables
create rule group_admin_permission_for_insert as on insert to group_admin_permission do nothing;

create table group_admin_binary_permission () inherits (group_admin_permission);
create table group_admin_read_write_permission () inherits (group_admin_permission);

-- required for bit_or aggregate
create or replace function intor(int,int) returns int language 'sql' immutable strict as '
select bitor($1::bit(32),$2::bit(32))::int';

-- aggregate for doing bitwise or operations on integers
create aggregate bit_or(basetype=int, sfunc=intor, stype=int, initcond=0);

-- per group permission query
create or replace view user_group_permission as
    select user_name, group_name, permission_item, permission_bits
    from group_admin
    join group_admin_permission using (group_admin_id)

      union all

    select user_name, group_name, permission_item, default_permission_bits
    from group_admin AS GA
    cross join permission_item AS PI
    where not exists(select * from group_admin_permission AS GAP
                     where GAP.group_admin_id = GA.group_admin_id
                     and GAP.permission_item = PI.permission_item);

-- aggregate (per user) permission query (with highest level of permissions)
create or replace view user_permission as
    select   user_name,
             permission_item,
             bit_or(permission_bits) as permission_bits
    from     user_group_permission
    group by user_name, permission_item;


-- Add message_group
create table message_group (
    message_id bigint not null,
    group_name text not null,
    constraint fk_message_group_message_id
        foreign key (message_id) references message (id)
        on delete cascade
);
create index message_group_message_id_index on message_group(message_id);

-- Add outbound flag for identifying message direction
alter table message add column outbound bool;
alter table message alter column outbound set default false;

COMMENT ON SCHEMA public IS 'PMX 5.301';

commit;
