
SET client_encoding = 'SQL_ASCII';
SET standard_conforming_strings = off;
SET check_function_bodies = false;
SET client_min_messages = warning;
SET escape_string_warning = off;

SET search_path = public, pg_catalog;

CREATE TYPE val_freq_t AS (
	val text,
	freq real
);

CREATE FUNCTION del_cfg_binval() RETURNS trigger
    AS '
declare del_change_id int;
begin
        del_change_id := nextval(''cfg_change_id_seq'');

        update cfg_instance set change_id = del_change_id
        where   instance_id = old.instance_id
        and     change_id < del_change_id;

        return old;
end'
    LANGUAGE plpgsql;

CREATE FUNCTION del_cfg_kv() RETURNS trigger
    AS '
declare del_change_id int;
begin
        del_change_id := nextval(''cfg_change_id_seq'');

        insert into cfg_item_change(instance_id, userkey, info1, info2, change_id, op)
                select old.instance_id, old.userkey, old.k_name, old.k_value::bytea,
                         del_change_id, ''D'';

        update cfg_instance set change_id = del_change_id
        where   instance_id = old.instance_id
        and     change_id < del_change_id;

        return old;
end'
    LANGUAGE plpgsql;

CREATE FUNCTION del_cfg_listrow() RETURNS trigger
    AS '
declare del_change_id int;
begin
        del_change_id := nextval(''cfg_change_id_seq'');

        insert into cfg_item_change(instance_id, userkey, info1, info2, change_id, op)
                select old.instance_id, old.userkey, old.list_value, ''''::bytea,
                        del_change_id, ''D'';

        update cfg_instance set change_id = del_change_id
        where   instance_id = old.instance_id
        and     change_id < del_change_id;

        return old;
end'
    LANGUAGE plpgsql;

CREATE FUNCTION del_prd_cfg_data() RETURNS void
    AS '
    delete from prd_applicable_dimension;
    delete from prd_dimension;
    delete from prd_direction;
    delete from prd_fact;
    delete from prd_nullified_dimension;
    delete from prd_reduction_action;
    delete from prd_runtime_fact_type;
    delete from prd_mta_action;
    delete from prd_policy_action;
'
    LANGUAGE sql;

CREATE FUNCTION delete_unmapped_rcpnts() RETURNS trigger
    AS '
    begin
        delete from recipient where id = old.recipient_id
        and not exists (
            select message_id from message_recipient
            where message_recipient.recipient_id = old.recipient_id
            and message_recipient.message_id != old.message_id
        );
        return old;
    end;
'
    LANGUAGE plpgsql;

CREATE FUNCTION floor_period(timestamptz, timestamptz, timestamptz, integer) RETURNS timestamptz
    AS '
    SELECT $2 + floor(extract(epoch from $1-$2)/secs)::int * secs * interval''1 sec''
    FROM (SELECT extract(epoch FROM range_interval($2,$3,$4)) AS secs) AS x;
'
    LANGUAGE sql IMMUTABLE STRICT;

CREATE FUNCTION generate_series(integer, integer, integer) RETURNS SETOF integer
    AS '
   DECLARE start ALIAS FOR $1;
           stop  ALIAS FOR $2;
           hop   ALIAS FOR $3;
           i     int;
   BEGIN
        i := start;
        WHILE i <= stop LOOP
                RETURN NEXT i;
                i := i + hop;
        END LOOP;
        RETURN;
   END
'
    LANGUAGE plpgsql IMMUTABLE;

CREATE FUNCTION generate_series(timestamptz, timestamptz, interval) RETURNS SETOF timestamptz
    AS '
   DECLARE _c timestamptz := $1;
   BEGIN
        WHILE _c <= $2 LOOP
           RETURN NEXT _c;
           _c := _c + $3;
        END LOOP;
   END;
'
    LANGUAGE plpgsql IMMUTABLE;

CREATE FUNCTION ins_cfg_binval() RETURNS trigger
    AS '
begin
        update cfg_instance set change_id = new.change_id
        where   instance_id = new.instance_id
        and     change_id < new.change_id;

        return new;
end'
    LANGUAGE plpgsql;

CREATE FUNCTION ins_cfg_instance() RETURNS trigger
    AS '
begin
        -- Ensure location_instance entries for all (location,instance_id)

        insert into cfg_location_instance(location_name, instance_id)
        select  location_name, new.instance_id
        from    cfg_location;

        return  new;
end'
    LANGUAGE plpgsql;

CREATE FUNCTION ins_cfg_kv() RETURNS trigger
    AS '
begin   insert into cfg_item_change(instance_id, userkey, info1, info2, change_id, op)
                select new.instance_id, new.userkey, new.k_name, new.k_value::bytea,
                 new.change_id, ''I'';

        update cfg_instance set change_id = new.change_id
        where   instance_id = new.instance_id
        and     change_id < new.change_id;

        return new;
end'
    LANGUAGE plpgsql;

CREATE FUNCTION ins_cfg_listrow() RETURNS trigger
    AS '
begin   insert into cfg_item_change(instance_id, userkey, info1, info2, change_id, op)
                select new.instance_id, new.userkey, new.list_value, ''''::bytea,
                        new.change_id, ''I'';

        update cfg_instance set change_id = new.change_id
        where   instance_id = new.instance_id
        and     change_id < new.change_id;

        return new;
end'
    LANGUAGE plpgsql;

CREATE FUNCTION ins_cfg_location() RETURNS trigger
    AS '
begin
        -- Ensure location_instance entries for all (location,instance_id)

        insert into cfg_location_instance(location_name, instance_id)
        select  new.location_name, instance_id
        from    cfg_instance;

        return  new;
end'
    LANGUAGE plpgsql;

CREATE FUNCTION ins_probability_ranges() RETURNS void
    AS '
declare
    granularity int := 2;
    bin_count int;
    bin int := 0;
    low int;
    high int;
begin
    bin_count = 100 / granularity;
    while bin < bin_count loop
        low := bin * granularity;
        high := (bin + 1) * granularity;
        insert into prd_probability_range (lo, hi) values (low, high);
        bin := bin + 1;
    end loop;
    return;
end'
    LANGUAGE plpgsql;

CREATE FUNCTION ins_size_ranges() RETURNS void
    AS '
declare
    exponent int := 0;
    magnitude constant int := 10;
    multiplier int;
    low bigint;
    high bigint;
begin
    while exponent <= magnitude loop
        multiplier := 1;
        while multiplier <= 9 loop
            low := (10 ^ exponent) * multiplier;
            high := ((10 ^ exponent) * (multiplier + 1)) - 1;
            insert into prd_msg_size_range (lo, hi) values (low, high);
            multiplier := multiplier + 1;
        end loop;
        exponent := exponent + 1;
    end loop;
    return;
end'
    LANGUAGE plpgsql;

CREATE FUNCTION ins_static_prd_cfg_data() RETURNS void
    AS '
    -- prd_user_action.id values chosen to match on-line encodings
    insert into prd_direction
        (name) values (''incoming'');
    insert into prd_direction
        (name) values (''outgoing'');
    insert into prd_dimension
        (name, is_purgeable) values (''recipient'', 1);
    insert into prd_dimension
        (name, is_purgeable) values (''sender'', 1);
    insert into prd_dimension
        (name, is_purgeable) values (''probability_range'', 0);
    insert into prd_dimension
        (name, is_purgeable) values (''msg_size_range'', 0);
    insert into prd_dimension
        (name, is_purgeable) values (''pmx_host'', 0);
    insert into prd_dimension
        (name, is_purgeable) values (''pmx_process'', 0);
    insert into prd_dimension
        (name, is_purgeable) values (''spam_virus_rule'', 1);
    insert into prd_dimension
        (name, is_purgeable) values (''period'', 1);
    insert into prd_dimension
        (name, is_purgeable) values (''fact_type'', 0);
    insert into prd_dimension
        (name, is_purgeable) values (''relay'', 1);
    insert into prd_dimension
        (name, is_purgeable) values (''rsrc_utlztn_range'', 0);
    insert into prd_dimension
        (name, is_purgeable) values (''concurrency_range'', 0);
    insert into prd_dimension
        (name, is_purgeable) values (''quarantine_reason'', 0);
    insert into prd_dimension
        (name, is_purgeable) values (''user_action'', 0);
    insert into prd_dimension
        (name, is_purgeable) values (''direction'', 0);
    insert into prd_dimension
        (name, is_purgeable) values (''atchmt_type'', 0);
    insert into prd_dimension
            (name, is_purgeable) values (''mta_action'', 0);
    insert into prd_dimension
            (name, is_purgeable) values (''policy_action'', 0);
    insert into prd_dimension
            (name, is_purgeable) values (''policy_reason'', 0);
    insert into prd_fact
        (name, is_collected) values (''msgs_processed_count'', 1);
    insert into prd_fact
        (name, is_collected) values (''atchmt_processed_count'', 1);
    insert into prd_fact
        (name, is_collected) values (''msgs_prcsd_per_rcpnt_count'', 1);
    insert into prd_fact
        (name, is_collected) values (''rule_hit_count'', 1);
    insert into prd_fact
        (name, is_collected) values (''user_action_count'', 1);
    insert into prd_fact
        (name, is_collected) values (''pmx_runtime_fact'', 1);
    insert into prd_fact
        (name, is_collected) values (''policy_mark_count'', 1);
    insert into prd_fact
        (name, is_collected) values (''mta_action_count'', 1);
    insert into prd_runtime_fact_type
        (id, name) values (1, ''disc_kbytes_written'');
    insert into prd_runtime_fact_type
        (id, name) values (2, ''disc_kbytes_read'');
    insert into prd_runtime_fact_type
        (id, name) values (3, ''disc_kbytes_allocated'');
    insert into prd_runtime_fact_type
        (id, name) values (4, ''ram_allocated'');
    insert into prd_runtime_fact_type
        (id, name) values (5, ''swap_allocated'');
    insert into prd_runtime_fact_type
        (id, name) values (6, ''cpu_utilized'');
    insert into prd_runtime_fact_type
        (id, name) values (7, ''load'');
    insert into prd_runtime_fact_type
        (id, name) values (8, ''uptime'');
    insert into prd_runtime_fact_type
        (id, name) values (9, ''total_quarantined_local'');
    insert into prd_runtime_fact_type
        (id, name) values (10, ''total_quarantined_central'');
    insert into prd_runtime_fact_type
        (id, name) values (11, ''new_quarantined_local'');
    insert into prd_runtime_fact_type
        (id, name) values (12, ''new_quarantined_central'');
    insert into prd_runtime_fact_type
        (id, name) values (13, ''quarantine_kbytes_local'');
    insert into prd_runtime_fact_type
        (id, name) values (14, ''quarantine_kbytes_central'');
    insert into prd_runtime_fact_type
        (id, name) values (15, ''disc_kbytes_free'');
    insert into prd_mta_action
        (name) values (''reject'');
    insert into prd_mta_action
        (name) values (''tempfail'');
    insert into prd_mta_action
        (name) values (''ok'');
    insert into prd_policy_action
        (name) values (''reject'');
    insert into prd_policy_action
        (name) values (''discard'');
    insert into prd_policy_action
        (name) values (''ok'');
'
    LANGUAGE sql;

CREATE FUNCTION ins_transient_prd_cfg_data() RETURNS void
    AS '
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''period'', ''msgs_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''pmx_host'', ''msgs_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''sender'', ''msgs_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''relay'', ''msgs_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''probability_range'', ''msgs_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''msg_size_range'', ''msgs_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''direction'', ''msgs_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''spam_virus_rule'', ''msgs_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''quarantine_reason'', ''msgs_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''period'', ''atchmt_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''pmx_host'', ''atchmt_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''msg_size_range'', ''atchmt_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''atchmt_type'', ''atchmt_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''period'', ''msgs_prcsd_per_rcpnt_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''pmx_host'', ''msgs_prcsd_per_rcpnt_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''quarantine_reason'', ''msgs_prcsd_per_rcpnt_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''recipient'', ''msgs_prcsd_per_rcpnt_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''period'', ''rule_hit_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''pmx_host'', ''rule_hit_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''spam_virus_rule'', ''rule_hit_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''period'', ''user_action_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''pmx_host'', ''user_action_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''recipient'', ''user_action_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''user_action'', ''user_action_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''relay'', ''user_action_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''sender'', ''user_action_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''probability_range'', ''user_action_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''period'', ''pmx_runtime_fact'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''fact_type'', ''pmx_runtime_fact'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''pmx_host'', ''pmx_runtime_fact'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''pmx_process'', ''pmx_runtime_fact'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''concurrency_range'', ''pmx_runtime_fact'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''rsrc_utlztn_range'', ''pmx_runtime_fact'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''period'', ''mta_action_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''pmx_host'', ''mta_action_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''mta_action'', ''mta_action_count'', 1);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
        recurrence, aggregation_period)
            values (''rule_hit_count'', ''rule_hit_count_expiry'',
                    ''expiry'', 172800, ''hourly'', null);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
        recurrence, aggregation_period)
            values (''policy_mark_count'', ''policy_mark_count_expiry'',
                    ''expiry'', 172800, ''hourly'', null);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
        recurrence, aggregation_period)
            values (''pmx_runtime_fact'', ''pmx_runtime_fact_aggregation'',
                    ''aggregation/avg'', 172800, ''hourly'', 3600);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
        recurrence, aggregation_period)
            values (''pmx_runtime_fact'', ''user_action_count_expiry'',
                    ''expiry'', 63115200, ''daily'', null);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
        recurrence, aggregation_period)
            values (''user_action_count'', ''user_action_count_aggregation'',
                    ''aggregation/sum'', 172800, ''hourly'', 3600);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
        recurrence, aggregation_period)
            values (''user_action_count'', ''user_action_count_expiry'',
                    ''expiry'', 31557600, ''daily'', null);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
         recurrence, aggregation_period)
            values (''msgs_processed_count'', ''msg_proc_expiry'',
                    ''expiry'', 63115200, ''daily'', null);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
        recurrence, aggregation_period)
            values (''msgs_processed_count'', ''msg_proc_aggregation'',
                    ''aggregation/sum'', 129600, ''hourly'', 3600);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
         recurrence, aggregation_period)
            values (''msgs_prcsd_per_rcpnt_count'', ''msg_proc_rcpnt_expiry'',
                    ''expiry'', 129600, ''hourly'', null);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
         recurrence, aggregation_period)
            values (''atchmt_processed_count'', ''atchmt_processed_expiry'',
                    ''expiry'', 129600, ''hourly'', null);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
        recurrence, aggregation_period)
            values (''mta_action_count'', ''mta_action_aggregation'',
                    ''aggregation/sum'', 172800, ''hourly'', 3600);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
        recurrence, aggregation_period)
            values (''mta_action_count'', ''mta_action_expiry'',
                    ''expiry'', 63115200, ''daily'', null);
    insert into prd_nullified_dimension
        (fact_name, action_name, dimension_name)
            values (''msgs_processed_count'', ''msg_proc_aggregation'', ''relay'');
    insert into prd_nullified_dimension
        (fact_name, action_name, dimension_name)
            values (''msgs_processed_count'', ''msg_proc_aggregation'', ''sender'');
'
    LANGUAGE sql;

CREATE FUNCTION intor(integer, integer) RETURNS integer
    AS '
select bitor($1::bit(32),$2::bit(32))::int'
    LANGUAGE sql IMMUTABLE STRICT;

CREATE FUNCTION period_seq(timestamptz, timestamptz, interval) RETURNS SETOF timestamptz
    AS '
    SELECT  $1 + secs*INTERVAL''1 sec'' AS period
    FROM    generate_series(0, EXTRACT(EPOCH FROM $2-$1)::int,
                            EXTRACT(EPOCH FROM $3)::int) AS seq(secs);
'
    LANGUAGE sql IMMUTABLE STRICT;

CREATE FUNCTION period_seq_grain(timestamptz, timestamptz, integer) RETURNS SETOF timestamptz
    AS '
    SELECT * FROM period_seq($1, $2, range_interval($1, $2, $3));
'
    LANGUAGE sql IMMUTABLE STRICT;

CREATE FUNCTION plpgsql_call_handler() RETURNS language_handler
    AS '/opt/pmx/postgres/lib/plpgsql', 'plpgsql_call_handler'
    LANGUAGE c;

CREATE FUNCTION range_interval(timestamptz, timestamptz, integer) RETURNS interval
    AS '
    SELECT
        CASE
            WHEN ($2 - $1)/$3 < INTERVAL ''1 hour'' THEN ''5 min''
            WHEN ($2 - $1)/$3 < INTERVAL ''4 hours'' THEN ''1 hour''
            WHEN ($2 - $1)/$3 < INTERVAL ''12 hours'' THEN ''4 hours''
            WHEN ($2 - $1)/$3 < INTERVAL ''1 day'' THEN ''12 hours''
            WHEN ($2 - $1)/$3 < INTERVAL ''2 days'' THEN ''1 day''
            WHEN ($2 - $1)/$3 < INTERVAL ''7 days'' THEN ''2 days''
            WHEN ($2 - $1)/$3 < INTERVAL ''2 weeks'' THEN ''1 week''
            WHEN ($2 - $1)/$3 < INTERVAL ''4 weeks'' THEN ''2 weeks''
            WHEN ($2 - $1)/$3 < INTERVAL ''8 weeks'' THEN ''4 weeks''
            ELSE ''8 weeks''
        END::interval;
'
    LANGUAGE sql IMMUTABLE STRICT;

CREATE FUNCTION sortable_mail_address(text) RETURNS text
    AS '
declare
        addrline  alias for $1;
        ret     text;
        pos   integer;
begin
        pos := position(''<'' in addrline);

        if pos > 0
        then    ret := substring(addrline from pos+1);
                pos := position(''>'' in ret);
        end if;

        if pos > 0
        then    ret := substring(ret from 1 for pos-1);
        else    ret := addrline;
        end if;

        return  ret;
end;'
    LANGUAGE plpgsql;

CREATE FUNCTION upd_cfg_binval() RETURNS trigger
    AS '
begin
        new.change_id := nextval(''cfg_change_id_seq'');

        update cfg_instance set change_id = new.change_id
        where   instance_id = new.instance_id
        and     change_id < new.change_id;

        return new;
end'
    LANGUAGE plpgsql;

CREATE FUNCTION upd_cfg_kv() RETURNS trigger
    AS '
begin
        -- This does NOT presume trigger FOR EACH ROW:
        new.change_id := nextval(''cfg_change_id_seq'');

        insert into cfg_item_change(instance_id, userkey, info1, info2, change_id, op)
                select new.instance_id, new.userkey, new.k_name,
                        new.k_value::bytea, new.change_id, ''U'';

        update cfg_instance set change_id = new.change_id
        where   instance_id = new.instance_id
        and     change_id < new.change_id;

        return new;
end'
    LANGUAGE plpgsql;

CREATE FUNCTION val_freq_set(text, text) RETURNS SETOF val_freq_t
    AS '
DECLARE tbl ALIAS FOR $1;
        col ALIAS FOR $2;
        vals        text[];
        freqs       real[];
        result      val_freq_t;
        i           int;
BEGIN
    SELECT  most_common_vals, most_common_freqs
    INTO    vals, freqs
    FROM    pg_stats
    WHERE   tablename = tbl AND attname = col;

    IF FOUND THEN
        FOR i IN array_lower(vals,1)..array_upper(vals,1) LOOP
            result.val  := vals[i];
            result.freq := freqs[i];
            RETURN NEXT result;
        END LOOP;
    END IF;
 
    RETURN;
END
'
    LANGUAGE plpgsql;

COMMENT ON FUNCTION val_freq_set(text, text) IS 'Returns most-common (val,freq) statistics from pg_stats, for a given table+column';

CREATE AGGREGATE bit_or(integer) (
    SFUNC = intor,
    STYPE = integer,
    INITCOND = '0'
);

SET default_tablespace = '';

SET default_with_oids = false;

CREATE TABLE permission (
    permission_name text NOT NULL,
    permission_bit integer NOT NULL
);

CREATE TABLE binary_permission (
)
INHERITS (permission);

CREATE TABLE permission_item (
    permission_item text NOT NULL,
    default_permission_bits integer DEFAULT 0 NOT NULL
);

CREATE TABLE binary_permission_item (
    default_permission_bits integer DEFAULT 1
)
INHERITS (permission_item);

CREATE TABLE cfg_alert (
    alert_id integer NOT NULL,
    location_id text,
    hostname text,
    ip text NOT NULL,
    ident text NOT NULL,
    priority integer DEFAULT 0,
    alert_time timestamp NOT NULL,
    info text NOT NULL,
    lang_id text NOT NULL,
    lang_args text[]
);

CREATE SEQUENCE cfg_alert_alert_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE cfg_alert_alert_id_seq OWNED BY cfg_alert.alert_id;

SELECT pg_catalog.setval('cfg_alert_alert_id_seq', 1, false);

CREATE TABLE cfg_change (
    id integer NOT NULL
);

CREATE SEQUENCE cfg_change_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE cfg_change_id_seq OWNED BY cfg_change.id;

SELECT pg_catalog.setval('cfg_change_id_seq', 1, true);

CREATE TABLE cfg_attribute (
    resource_name text NOT NULL,
    attribute_name text NOT NULL,
    attribute_value text NOT NULL,
    change_id integer DEFAULT nextval('cfg_change_id_seq'::regclass) NOT NULL
);

CREATE TABLE cfg_binval (
    instance_id integer NOT NULL,
    b_value bytea,
    change_id integer DEFAULT nextval('cfg_change_id_seq'::regclass) NOT NULL
);

CREATE TABLE cfg_enduser (
    enduser_id integer NOT NULL,
    enduser_name text NOT NULL,
    change_id integer DEFAULT nextval('cfg_change_id_seq'::regclass) NOT NULL
);

CREATE SEQUENCE cfg_enduser_enduser_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE cfg_enduser_enduser_id_seq OWNED BY cfg_enduser.enduser_id;

SELECT pg_catalog.setval('cfg_enduser_enduser_id_seq', 1, false);

CREATE TABLE cfg_instance (
    instance_id integer NOT NULL,
    resource_name text NOT NULL,
    profile_id integer NOT NULL,
    instance_value text,
    change_id integer DEFAULT nextval('cfg_change_id_seq'::regclass) NOT NULL
);

CREATE SEQUENCE cfg_instance_instance_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE cfg_instance_instance_id_seq OWNED BY cfg_instance.instance_id;

SELECT pg_catalog.setval('cfg_instance_instance_id_seq', 1, false);

CREATE TABLE cfg_item_change (
    instance_id integer NOT NULL,
    userkey text DEFAULT '',
    info1 text NOT NULL,
    info2 bytea NOT NULL,
    change_id integer NOT NULL,
    op character(1)
);

CREATE TABLE cfg_kv (
    instance_id integer NOT NULL,
    userkey text DEFAULT '' NOT NULL,
    k_name text NOT NULL,
    k_value bytea NOT NULL,
    change_id integer DEFAULT nextval('cfg_change_id_seq'::regclass) NOT NULL
);

CREATE TABLE cfg_listrow (
    instance_id integer NOT NULL,
    userkey text DEFAULT '',
    list_value text NOT NULL,
    change_id integer DEFAULT nextval('cfg_change_id_seq'::regclass) NOT NULL
);

CREATE TABLE cfg_location (
    location_id integer NOT NULL,
    location_name text NOT NULL,
    hostname text NOT NULL,
    ip inet NOT NULL,
    manager_protocol varchar(255) DEFAULT 'https'::varchar NOT NULL,
    manager_port integer DEFAULT 18080 NOT NULL,
    pong timestamp,
    profile_id integer DEFAULT 0 NOT NULL,
    last_change_id integer DEFAULT 0 NOT NULL,
    next_change_id integer DEFAULT 0 NOT NULL
);

CREATE TABLE cfg_location_instance (
    location_name text NOT NULL,
    instance_id integer NOT NULL,
    last_change_id integer DEFAULT 0 NOT NULL
);

CREATE SEQUENCE cfg_location_location_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE cfg_location_location_id_seq OWNED BY cfg_location.location_id;

SELECT pg_catalog.setval('cfg_location_location_id_seq', 1, false);

CREATE TABLE cfg_profile (
    profile_id integer NOT NULL,
    profile_name text NOT NULL,
    change_id integer DEFAULT nextval('cfg_change_id_seq'::regclass) NOT NULL
);

CREATE TABLE cfg_resource (
    resource_name text NOT NULL,
    resource_type text NOT NULL,
    change_id integer DEFAULT nextval('cfg_change_id_seq'::regclass) NOT NULL
);

CREATE TABLE digest_state (
    digest_user varchar(255) NOT NULL,
    digest_type varchar(32) NOT NULL,
    last_m_date timestamptz,
    last_d_date timestamptz
);

CREATE TABLE group_admin (
    group_admin_id integer NOT NULL,
    group_name text NOT NULL,
    user_name text NOT NULL
);

CREATE TABLE group_admin_permission (
    group_admin_permission_id integer NOT NULL,
    group_admin_id integer NOT NULL,
    permission_item text NOT NULL,
    permission_bits integer DEFAULT 0 NOT NULL
);

CREATE SEQUENCE group_admin_permission_group_admin_permission_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE group_admin_permission_group_admin_permission_id_seq OWNED BY group_admin_permission.group_admin_permission_id;

SELECT pg_catalog.setval('group_admin_permission_group_admin_permission_id_seq', 1, false);

CREATE TABLE group_admin_binary_permission (
)
INHERITS (group_admin_permission);

CREATE SEQUENCE group_admin_group_admin_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE group_admin_group_admin_id_seq OWNED BY group_admin.group_admin_id;

SELECT pg_catalog.setval('group_admin_group_admin_id_seq', 1, false);

CREATE TABLE group_admin_read_write_permission (
)
INHERITS (group_admin_permission);

CREATE TABLE group_multifile (
    group_multifile_id integer NOT NULL,
    multifile_id text NOT NULL,
    group_name text NOT NULL,
    content text DEFAULT '' NOT NULL,
    checksum text DEFAULT '' NOT NULL
);

CREATE SEQUENCE group_multifile_group_multifile_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE group_multifile_group_multifile_id_seq OWNED BY group_multifile.group_multifile_id;

SELECT pg_catalog.setval('group_multifile_group_multifile_id_seq', 1, false);

CREATE TABLE group_policy_option_opt_in (
    name text NOT NULL,
    description text NOT NULL,
    default_on boolean NOT NULL,
    opt_in boolean NOT NULL,
    list_name text NOT NULL
);

CREATE TABLE lock (
    capability varchar(255) NOT NULL,
    running integer NOT NULL
);

CREATE TABLE m_reason (
    reason_id integer NOT NULL,
    name varchar(255) NOT NULL
);

CREATE SEQUENCE m_reason_reason_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE m_reason_reason_id_seq OWNED BY m_reason.reason_id;

SELECT pg_catalog.setval('m_reason_reason_id_seq', 3, true);

CREATE TABLE message (
    id bigint NOT NULL,
    m_global_id varchar(255) NOT NULL,
    reason_id smallint NOT NULL,
    location_name varchar(255) NOT NULL,
    m_date timestamp,
    m_queue_id varchar(255),
    h_message_id varchar(255),
    c_subject_utf8 varchar(255),
    c_subject_body_utf8 varchar(255),
    env_from_local varchar(255),
    env_from_domain varchar(255),
    h_from_local varchar(255),
    h_from_domain varchar(255),
    h_from varchar(255),
    h_to varchar(255),
    m_milter_host varchar(255),
    m_relay varchar(255),
    m_spam_probability double precision,
    m_message_size integer,
    m_header_size integer,
    outbound boolean DEFAULT false,
    reason text
);

CREATE TABLE message_body (
    message_id bigint NOT NULL,
    c_body varchar(2000) NOT NULL,
    raw_msg varchar(4000) NOT NULL
);

CREATE TABLE message_group (
    message_id bigint NOT NULL,
    group_name text NOT NULL
);

CREATE SEQUENCE message_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE message_id_seq OWNED BY message.id;

SELECT pg_catalog.setval('message_id_seq', 1, false);

CREATE TABLE message_recipient (
    recipient_id bigint NOT NULL,
    message_id bigint NOT NULL,
    is_mapped smallint DEFAULT 0 NOT NULL,
    is_calculated smallint DEFAULT 0 NOT NULL,
    is_envelope smallint DEFAULT 0 NOT NULL,
    reason_id smallint NOT NULL,
    action smallint
);

CREATE TABLE multifile (
    multifile_id text NOT NULL,
    charset text DEFAULT 'latin1' NOT NULL,
    name text NOT NULL,
    type text NOT NULL
);

CREATE TABLE pmdb_schema_event (
    event_type varchar(255) NOT NULL,
    event_date timestamp NOT NULL,
    version varchar(255) NOT NULL,
    comment varchar(255)
);

CREATE TABLE pmx_forward_config (
    access_right text NOT NULL,
    address text NOT NULL,
    subject text NOT NULL,
    description text NOT NULL
);

CREATE TABLE pmx_group (
    group_name text NOT NULL,
    long_name text NOT NULL,
    CONSTRAINT chk_pmx_group_group_name
            CHECK (group_name ~ '^[a-z0-9][a-z0-9_-]+$')
);

CREATE TABLE pmx_user (
    user_name text NOT NULL,
    passphrase text NOT NULL,
    email text NOT NULL,
    fullname text,
    locale text,
    super_user boolean DEFAULT false,
    CONSTRAINT chk_pmx_user_user_name
            CHECK (user_name ~ '^[a-z][a-z0-9_.-]+$')
);

CREATE TABLE prd_applicable_dimension (
    dimension_name varchar(32) NOT NULL,
    fact_name varchar(32) NOT NULL,
    is_collected integer NOT NULL,
    CONSTRAINT ck_prd_app_dmnsn_is_clctd
            CHECK (is_collected = ANY (ARRAY[0, 1]))
);

CREATE TABLE prd_atchmt_processed_count (
    period_id integer NOT NULL,
    pmx_host_id integer NOT NULL,
    msg_size_range_id integer NOT NULL,
    atchmt_type_id integer NOT NULL,
    sender_id integer,
    relay_id integer,
    direction_id smallint,
    tally integer
);

CREATE TABLE prd_atchmt_type (
    id integer NOT NULL,
    name varchar(64) NOT NULL
);

CREATE SEQUENCE prd_atchmt_type_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE prd_atchmt_type_id_seq OWNED BY prd_atchmt_type.id;

SELECT pg_catalog.setval('prd_atchmt_type_id_seq', 1, false);

CREATE TABLE prd_concurrency_range (
    id integer NOT NULL,
    hi integer NOT NULL,
    lo integer NOT NULL
);

CREATE SEQUENCE prd_concurrency_range_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE prd_concurrency_range_id_seq OWNED BY prd_concurrency_range.id;

SELECT pg_catalog.setval('prd_concurrency_range_id_seq', 1, false);

CREATE TABLE prd_connections_by_action (
    group_name text NOT NULL,
    period_start timestamptz NOT NULL,
    action text NOT NULL,
    tally integer NOT NULL
);

CREATE TABLE prd_custom_mark_hits (
    group_name text NOT NULL,
    period_start timestamptz NOT NULL,
    mark_key integer NOT NULL,
    mark_val text NOT NULL,
    tally integer NOT NULL
);

CREATE TABLE prd_dimension (
    name varchar(32) NOT NULL,
    is_purgeable smallint NOT NULL,
    match_pattern varchar(256),
    dont_match_pattern varchar(256),
    increment integer,
    scale_type varchar(16),
    CONSTRAINT ck_prd_dimension_is_purgeable
            CHECK (is_purgeable = ANY (ARRAY[0, 1])),
    CONSTRAINT ck_prd_dimension_scale_type
            CHECK (scale_type::text = ANY (ARRAY['linear', 'log2', 'log10']))
);

CREATE TABLE prd_direction (
    id integer NOT NULL,
    name varchar(16) NOT NULL
);

CREATE SEQUENCE prd_direction_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE prd_direction_id_seq OWNED BY prd_direction.id;

SELECT pg_catalog.setval('prd_direction_id_seq', 2, true);

CREATE TABLE prd_fact (
    name varchar(32) NOT NULL,
    is_collected integer NOT NULL,
    CONSTRAINT ck_prd_fact_is_collected
            CHECK (is_collected = ANY (ARRAY[0, 1]))
);

CREATE TABLE prd_msg_size_range (
    id integer NOT NULL,
    lo bigint NOT NULL,
    hi bigint NOT NULL
);

CREATE SEQUENCE prd_msg_size_range_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE prd_msg_size_range_id_seq OWNED BY prd_msg_size_range.id;

SELECT pg_catalog.setval('prd_msg_size_range_id_seq', 99, true);

CREATE TABLE prd_msgs_by_policy_block (
    group_name text NOT NULL,
    period_start timestamptz NOT NULL,
    blocked boolean NOT NULL,
    tally integer NOT NULL
);

CREATE TABLE prd_msgs_by_type (
    group_name text NOT NULL,
    period_start timestamptz NOT NULL,
    type text NOT NULL,
    inbound boolean NOT NULL,
    tally integer NOT NULL
);

CREATE TABLE prd_msgs_by_virus (
    group_name text NOT NULL,
    period_start timestamptz NOT NULL,
    virus_name text NOT NULL,
    inbound boolean NOT NULL,
    tally integer NOT NULL
);

CREATE TABLE prd_msgs_prcsd_per_rcpnt_count (
    period_id integer NOT NULL,
    pmx_host_id integer NOT NULL,
    recipient_id integer NOT NULL,
    tally integer NOT NULL,
    quarantine_reason_id integer
);

CREATE TABLE prd_msgs_processed_count (
    period_id integer NOT NULL,
    pmx_host_id integer NOT NULL,
    sender_id integer,
    relay_id integer,
    probability_range_id integer,
    quarantine_reason_id integer,
    msg_size_range_id integer,
    spam_virus_rule_id integer,
    direction_id smallint,
    policy_reason_id integer,
    policy_action_id integer,
    tally integer
);

CREATE TABLE prd_period (
    id integer NOT NULL,
    period_start timestamptz NOT NULL,
    period_end timestamptz NOT NULL,
    duration integer NOT NULL,
    super_period_start timestamptz NOT NULL
);

CREATE TABLE prd_pmx_host (
    id integer NOT NULL,
    name varchar(128) NOT NULL
);

CREATE TABLE prd_quarantine_reason (
    id integer NOT NULL,
    name varchar(16) NOT NULL
);

CREATE VIEW prd_msgs_processed_count_per_reason_view AS
  SELECT prd_period.id AS period_id, prd_quarantine_reason.id AS quarantine_reason_id, prd_pmx_host.id AS pmx_host_id, COALESCE(prd_msgs_processed_count.tally, 0) AS tally, prd_period.duration, prd_direction.name AS direction, prd_period.period_start, prd_period.period_end
    FROM prd_period
   CROSS JOIN prd_pmx_host
   CROSS JOIN prd_quarantine_reason
   CROSS JOIN prd_direction
    LEFT JOIN prd_msgs_processed_count ON prd_period.id = prd_msgs_processed_count.period_id AND prd_quarantine_reason.id = prd_msgs_processed_count.quarantine_reason_id AND prd_pmx_host.id = prd_msgs_processed_count.pmx_host_id AND prd_direction.id = prd_msgs_processed_count.direction_id;

CREATE TABLE prd_mta_action (
    id integer NOT NULL,
    name varchar(64) NOT NULL
);

CREATE TABLE prd_mta_action_count (
    period_id integer NOT NULL,
    pmx_host_id integer NOT NULL,
    mta_action_id integer NOT NULL,
    tally integer
);

CREATE VIEW prd_mta_action_count_view AS
  SELECT prd_period.id AS period_id, prd_pmx_host.id AS pmx_host_id, prd_mta_action.id AS mta_action_id, prd_period.period_start, prd_period.period_end, prd_period.duration, COALESCE(grouped_action_count.tally, 0::bigint) AS tally
    FROM prd_period
   CROSS JOIN prd_mta_action
   CROSS JOIN prd_pmx_host
    LEFT JOIN ( SELECT action_count.period_id, action_count.pmx_host_id, action_count.mta_action_name, sum(action_count.tally) AS tally
    FROM ( SELECT prd_mta_action_count.period_id, prd_mta_action_count.pmx_host_id, 
                 CASE
                     WHEN prd_mta_action.name::text = 'ok' THEN 'ok'
                     ELSE 'reject'
                 END AS mta_action_name, prd_mta_action_count.tally
            FROM prd_mta_action_count
       JOIN prd_mta_action ON prd_mta_action.id = prd_mta_action_count.mta_action_id) action_count
   GROUP BY action_count.period_id, action_count.pmx_host_id, action_count.mta_action_name) grouped_action_count ON grouped_action_count.period_id = prd_period.id AND grouped_action_count.mta_action_name = prd_mta_action.name::text AND grouped_action_count.pmx_host_id = prd_pmx_host.id
   WHERE prd_mta_action.name::text = 'ok' OR prd_mta_action.name::text = 'reject';

CREATE SEQUENCE prd_mta_action_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE prd_mta_action_id_seq OWNED BY prd_mta_action.id;

SELECT pg_catalog.setval('prd_mta_action_id_seq', 3, true);

CREATE TABLE prd_nullified_dimension (
    fact_name varchar(32) NOT NULL,
    action_name varchar(128) NOT NULL,
    dimension_name varchar(32) NOT NULL
);

CREATE SEQUENCE prd_period_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE prd_period_id_seq OWNED BY prd_period.id;

SELECT pg_catalog.setval('prd_period_id_seq', 1, false);

CREATE SEQUENCE prd_pmx_host_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE prd_pmx_host_id_seq OWNED BY prd_pmx_host.id;

SELECT pg_catalog.setval('prd_pmx_host_id_seq', 1, false);

CREATE TABLE prd_pmx_process (
    id integer NOT NULL,
    name varchar(128) NOT NULL
);

CREATE SEQUENCE prd_pmx_process_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE prd_pmx_process_id_seq OWNED BY prd_pmx_process.id;

SELECT pg_catalog.setval('prd_pmx_process_id_seq', 1, false);

CREATE TABLE prd_pmx_runtime_fact (
    period_id integer NOT NULL,
    runtime_fact_type_id smallint NOT NULL,
    pmx_host_id integer NOT NULL,
    pmx_process_id integer,
    tally integer,
    value_absolute bigint,
    value_relative bigint
);

CREATE TABLE prd_runtime_fact_type (
    id integer NOT NULL,
    name varchar(64) NOT NULL
);

CREATE VIEW prd_pmx_runtime_fact_quarantine_view AS
  SELECT prd_period.id AS period_id, prd_period.period_start, prd_period.period_end, prd_period.duration, prd_runtime_fact_type.id AS runtime_fact_type_id, prd_pmx_host.id AS pmx_host_id, COALESCE(prd_pmx_runtime_fact_2.value_absolute, 0::bigint) AS value_absolute
    FROM prd_period
    JOIN prd_runtime_fact_type ON (EXISTS ( SELECT 'x'
       FROM prd_pmx_runtime_fact
      WHERE prd_pmx_runtime_fact.period_id = prd_period.id AND (prd_runtime_fact_type.name::text = 'total_quarantined_local' OR prd_runtime_fact_type.name::text = 'quarantine_kbytes_local' OR prd_runtime_fact_type.name::text = 'new_quarantined_local')))
   CROSS JOIN prd_pmx_host
    LEFT JOIN prd_pmx_runtime_fact prd_pmx_runtime_fact_2 ON prd_period.id = prd_pmx_runtime_fact_2.period_id AND prd_pmx_runtime_fact_2.runtime_fact_type_id = prd_runtime_fact_type.id AND (prd_pmx_runtime_fact_2.runtime_fact_type_id IN ( SELECT prd_runtime_fact_type_2.id
    FROM prd_runtime_fact_type prd_runtime_fact_type_2
   WHERE (prd_runtime_fact_type_2.name::text = 'total_quarantined_local' OR prd_runtime_fact_type_2.name::text = 'quarantine_kbytes_local' OR prd_runtime_fact_type_2.name::text = 'new_quarantined_local') AND prd_pmx_host.id = prd_pmx_runtime_fact_2.pmx_host_id));

CREATE TABLE prd_policy_action (
    id integer NOT NULL,
    name varchar(64) NOT NULL
);

CREATE VIEW prd_policy_action_count_view AS
  SELECT prd_period.id AS period_id, prd_pmx_host.id AS pmx_host_id, prd_policy_action.id AS policy_action_id, prd_period.period_start, prd_period.period_end, prd_period.duration, COALESCE(grouped_action_count.tally, 0::bigint) AS tally
    FROM prd_period
   CROSS JOIN prd_policy_action
   CROSS JOIN prd_pmx_host
    LEFT JOIN ( SELECT action_count.period_id, action_count.pmx_host_id, action_count.policy_action_name, sum(action_count.tally) AS tally
    FROM ( SELECT prd_msgs_processed_count.period_id, prd_msgs_processed_count.pmx_host_id, 
                 CASE
                     WHEN prd_policy_action.name::text = 'ok' THEN 'ok'
                     ELSE 'reject'
                 END AS policy_action_name, prd_msgs_processed_count.tally
            FROM prd_msgs_processed_count
       JOIN prd_policy_action ON prd_policy_action.id = prd_msgs_processed_count.policy_action_id) action_count
   GROUP BY action_count.period_id, action_count.pmx_host_id, action_count.policy_action_name) grouped_action_count ON grouped_action_count.period_id = prd_period.id AND grouped_action_count.policy_action_name = prd_policy_action.name::text AND grouped_action_count.pmx_host_id = prd_pmx_host.id
   WHERE prd_policy_action.name::text = 'ok' OR prd_policy_action.name::text = 'reject';

CREATE SEQUENCE prd_policy_action_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE prd_policy_action_id_seq OWNED BY prd_policy_action.id;

SELECT pg_catalog.setval('prd_policy_action_id_seq', 3, true);

CREATE TABLE prd_policy_mark (
    id integer NOT NULL,
    name varchar(64) NOT NULL
);

CREATE TABLE prd_policy_mark_count (
    period_id integer NOT NULL,
    pmx_host_id integer NOT NULL,
    policy_mark_id integer,
    tally integer
);

CREATE TABLE prd_policy_mark_hits (
    group_name text NOT NULL,
    period_start timestamptz NOT NULL,
    policy_mark text NOT NULL,
    tally integer NOT NULL
);

CREATE SEQUENCE prd_policy_mark_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE prd_policy_mark_id_seq OWNED BY prd_policy_mark.id;

SELECT pg_catalog.setval('prd_policy_mark_id_seq', 1, false);

CREATE TABLE prd_policy_reason (
    id integer NOT NULL,
    name varchar(64) NOT NULL
);

CREATE SEQUENCE prd_policy_reason_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE prd_policy_reason_id_seq OWNED BY prd_policy_reason.id;

SELECT pg_catalog.setval('prd_policy_reason_id_seq', 1, false);

CREATE TABLE prd_probability_range (
    id integer NOT NULL,
    lo integer NOT NULL,
    hi integer NOT NULL
);

CREATE SEQUENCE prd_probability_range_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE prd_probability_range_id_seq OWNED BY prd_probability_range.id;

SELECT pg_catalog.setval('prd_probability_range_id_seq', 50, true);

CREATE SEQUENCE prd_quarantine_reason_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE prd_quarantine_reason_id_seq OWNED BY prd_quarantine_reason.id;

SELECT pg_catalog.setval('prd_quarantine_reason_id_seq', 1, false);

CREATE TABLE prd_recipient (
    id integer NOT NULL,
    local varchar(256) NOT NULL,
    domain varchar(256)
);

CREATE SEQUENCE prd_recipient_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE prd_recipient_id_seq OWNED BY prd_recipient.id;

SELECT pg_catalog.setval('prd_recipient_id_seq', 1, false);

CREATE TABLE prd_reduction_action (
    fact_name varchar(32) NOT NULL,
    action_name varchar(128) NOT NULL,
    action_type varchar(32) NOT NULL,
    age_threshold integer NOT NULL,
    recurrence varchar(32) NOT NULL,
    aggregation_period integer,
    CONSTRAINT ck_prd_rdctn_actn_action_type
            CHECK (action_type::text = ANY (ARRAY['expiry', 'aggregation/sum', 'aggregation/avg'])),
    CONSTRAINT ck_prd_rdctn_actn_recurrence
            CHECK (recurrence::text = ANY (ARRAY['hourly', 'daily', 'weekly']))
);

CREATE TABLE prd_relay (
    id integer NOT NULL,
    address varchar(256) NOT NULL
);

CREATE SEQUENCE prd_relay_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE prd_relay_id_seq OWNED BY prd_relay.id;

SELECT pg_catalog.setval('prd_relay_id_seq', 1, false);

CREATE TABLE prd_relays_by_type (
    group_name text NOT NULL,
    period_start timestamptz NOT NULL,
    type text NOT NULL,
    relay text NOT NULL,
    inbound boolean NOT NULL,
    tally integer NOT NULL
);

CREATE TABLE prd_rsrc_utlztn_range (
    id integer NOT NULL,
    type varchar(64) NOT NULL,
    lo integer NOT NULL,
    hi integer NOT NULL,
    CONSTRAINT ck_prd_rsrc_utlztn_range_type
            CHECK (type::text = ANY (ARRAY['cpu', 'ram', 'virtual', 'disk_space', 'disk_rate']))
);

CREATE SEQUENCE prd_rsrc_utlztn_range_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE prd_rsrc_utlztn_range_id_seq OWNED BY prd_rsrc_utlztn_range.id;

SELECT pg_catalog.setval('prd_rsrc_utlztn_range_id_seq', 1, false);

CREATE TABLE prd_rule_hit_count (
    period_id integer NOT NULL,
    pmx_host_id integer NOT NULL,
    spam_virus_rule_id integer NOT NULL,
    tally integer NOT NULL
);

CREATE SEQUENCE prd_runtime_fact_type_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE prd_runtime_fact_type_id_seq OWNED BY prd_runtime_fact_type.id;

SELECT pg_catalog.setval('prd_runtime_fact_type_id_seq', 1, false);

CREATE TABLE prd_sender (
    id integer NOT NULL,
    local varchar(256) NOT NULL,
    domain varchar(256)
);

CREATE SEQUENCE prd_sender_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE prd_sender_id_seq OWNED BY prd_sender.id;

SELECT pg_catalog.setval('prd_sender_id_seq', 1, false);

CREATE TABLE prd_senders_by_type (
    group_name text NOT NULL,
    period_start timestamptz NOT NULL,
    type text NOT NULL,
    sender text NOT NULL,
    inbound boolean NOT NULL,
    tally integer NOT NULL
);

CREATE TABLE prd_spam_by_recipient (
    group_name text NOT NULL,
    period_start timestamptz NOT NULL,
    recipient text NOT NULL,
    tally integer NOT NULL
);

CREATE TABLE prd_spam_virus_rule (
    id integer NOT NULL,
    type varchar(16) NOT NULL,
    rule varchar(256) NOT NULL,
    CONSTRAINT ck_prd_spam_virus_rule
            CHECK (type::text = ANY (ARRAY['spam', 'virus']))
);

CREATE SEQUENCE prd_spam_virus_rule_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE prd_spam_virus_rule_id_seq OWNED BY prd_spam_virus_rule.id;

SELECT pg_catalog.setval('prd_spam_virus_rule_id_seq', 1, false);

CREATE TABLE prd_state (
    name text NOT NULL,
    asof timestamptz DEFAULT '1969-12-31 16:00:00-08'::timestamptz NOT NULL
);

CREATE TABLE prd_user_action (
    id smallint NOT NULL,
    name varchar(16) NOT NULL
);

CREATE TABLE prd_user_action_count (
    period_id integer NOT NULL,
    pmx_host_id integer NOT NULL,
    recipient_id integer NOT NULL,
    user_action_id smallint NOT NULL,
    tally integer NOT NULL,
    probability_range_id integer,
    relay_id integer,
    sender_id integer
);

CREATE TABLE qrntn_action_request (
    submission_time timestamptz DEFAULT now(),
    m_global_id varchar(255) NOT NULL,
    location_name varchar(255) NOT NULL,
    action smallint DEFAULT 0 NOT NULL,
    local varchar(255),
    domain varchar(255),
    attempt_count integer DEFAULT 0 NOT NULL
);

CREATE TABLE quarantine_rule (
    id bigint NOT NULL,
    rule_text varchar(255) NOT NULL,
    reason_id smallint NOT NULL
);

CREATE TABLE quarantine_rule_hit (
    message_id bigint NOT NULL,
    quarantine_rule_id bigint NOT NULL
);

CREATE SEQUENCE quarantine_rule_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE quarantine_rule_id_seq OWNED BY quarantine_rule.id;

SELECT pg_catalog.setval('quarantine_rule_id_seq', 1, false);

CREATE TABLE read_write_permission (
)
INHERITS (permission);

CREATE TABLE read_write_permission_item (
    default_permission_bits integer DEFAULT 6
)
INHERITS (permission_item);

CREATE TABLE recipient (
    id bigint NOT NULL,
    local varchar(255) NOT NULL,
    domain varchar(255) NOT NULL
);

CREATE SEQUENCE recipient_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE recipient_id_seq OWNED BY recipient.id;

SELECT pg_catalog.setval('recipient_id_seq', 1, false);

CREATE TABLE session (
    id character(32) NOT NULL,
    a_session text NOT NULL
);

CREATE VIEW user_group_permission AS
  SELECT group_admin.user_name, group_admin.group_name, group_admin_permission.permission_item, group_admin_permission.permission_bits
    FROM group_admin
    JOIN group_admin_permission USING (group_admin_id)
 UNION ALL 
  SELECT ga.user_name, ga.group_name, pi.permission_item, pi.default_permission_bits AS permission_bits
    FROM group_admin ga
   CROSS JOIN permission_item pi
   WHERE NOT (EXISTS ( SELECT gap.group_admin_permission_id, gap.group_admin_id, gap.permission_item, gap.permission_bits
       FROM group_admin_permission gap
      WHERE gap.group_admin_id = ga.group_admin_id AND gap.permission_item = pi.permission_item));

CREATE VIEW user_permission AS
  SELECT user_group_permission.user_name, user_group_permission.permission_item, bit_or(user_group_permission.permission_bits) AS permission_bits
    FROM user_group_permission
   GROUP BY user_group_permission.user_name, user_group_permission.permission_item;

ALTER TABLE cfg_alert ALTER COLUMN alert_id SET DEFAULT nextval('cfg_alert_alert_id_seq'::regclass);

ALTER TABLE cfg_change ALTER COLUMN id SET DEFAULT nextval('cfg_change_id_seq'::regclass);

ALTER TABLE cfg_enduser ALTER COLUMN enduser_id SET DEFAULT nextval('cfg_enduser_enduser_id_seq'::regclass);

ALTER TABLE cfg_instance ALTER COLUMN instance_id SET DEFAULT nextval('cfg_instance_instance_id_seq'::regclass);

ALTER TABLE cfg_location ALTER COLUMN location_id SET DEFAULT nextval('cfg_location_location_id_seq'::regclass);

ALTER TABLE group_admin ALTER COLUMN group_admin_id SET DEFAULT nextval('group_admin_group_admin_id_seq'::regclass);

ALTER TABLE group_admin_permission ALTER COLUMN group_admin_permission_id SET DEFAULT nextval('group_admin_permission_group_admin_permission_id_seq'::regclass);

ALTER TABLE group_multifile ALTER COLUMN group_multifile_id SET DEFAULT nextval('group_multifile_group_multifile_id_seq'::regclass);

ALTER TABLE m_reason ALTER COLUMN reason_id SET DEFAULT nextval('m_reason_reason_id_seq'::regclass);

ALTER TABLE message ALTER COLUMN id SET DEFAULT nextval('message_id_seq'::regclass);

ALTER TABLE prd_atchmt_type ALTER COLUMN id SET DEFAULT nextval('prd_atchmt_type_id_seq'::regclass);

ALTER TABLE prd_concurrency_range ALTER COLUMN id SET DEFAULT nextval('prd_concurrency_range_id_seq'::regclass);

ALTER TABLE prd_direction ALTER COLUMN id SET DEFAULT nextval('prd_direction_id_seq'::regclass);

ALTER TABLE prd_msg_size_range ALTER COLUMN id SET DEFAULT nextval('prd_msg_size_range_id_seq'::regclass);

ALTER TABLE prd_mta_action ALTER COLUMN id SET DEFAULT nextval('prd_mta_action_id_seq'::regclass);

ALTER TABLE prd_period ALTER COLUMN id SET DEFAULT nextval('prd_period_id_seq'::regclass);

ALTER TABLE prd_pmx_host ALTER COLUMN id SET DEFAULT nextval('prd_pmx_host_id_seq'::regclass);

ALTER TABLE prd_pmx_process ALTER COLUMN id SET DEFAULT nextval('prd_pmx_process_id_seq'::regclass);

ALTER TABLE prd_policy_action ALTER COLUMN id SET DEFAULT nextval('prd_policy_action_id_seq'::regclass);

ALTER TABLE prd_policy_mark ALTER COLUMN id SET DEFAULT nextval('prd_policy_mark_id_seq'::regclass);

ALTER TABLE prd_policy_reason ALTER COLUMN id SET DEFAULT nextval('prd_policy_reason_id_seq'::regclass);

ALTER TABLE prd_probability_range ALTER COLUMN id SET DEFAULT nextval('prd_probability_range_id_seq'::regclass);

ALTER TABLE prd_quarantine_reason ALTER COLUMN id SET DEFAULT nextval('prd_quarantine_reason_id_seq'::regclass);

ALTER TABLE prd_recipient ALTER COLUMN id SET DEFAULT nextval('prd_recipient_id_seq'::regclass);

ALTER TABLE prd_relay ALTER COLUMN id SET DEFAULT nextval('prd_relay_id_seq'::regclass);

ALTER TABLE prd_rsrc_utlztn_range ALTER COLUMN id SET DEFAULT nextval('prd_rsrc_utlztn_range_id_seq'::regclass);

ALTER TABLE prd_runtime_fact_type ALTER COLUMN id SET DEFAULT nextval('prd_runtime_fact_type_id_seq'::regclass);

ALTER TABLE prd_sender ALTER COLUMN id SET DEFAULT nextval('prd_sender_id_seq'::regclass);

ALTER TABLE prd_spam_virus_rule ALTER COLUMN id SET DEFAULT nextval('prd_spam_virus_rule_id_seq'::regclass);

ALTER TABLE quarantine_rule ALTER COLUMN id SET DEFAULT nextval('quarantine_rule_id_seq'::regclass);

ALTER TABLE recipient ALTER COLUMN id SET DEFAULT nextval('recipient_id_seq'::regclass);

COPY binary_permission (permission_name, permission_bit) FROM stdin;
off	0
on	1
\.

COPY binary_permission_item (permission_item, default_permission_bits) FROM stdin;
help	1
configuration	1
quarantine	1
quarantine.actions	1
quarantine.actions.approve	1
quarantine.actions.delete	1
quarantine.actions.forward	1
quarantine.actions.save	1
quarantine.preview	1
quarantine.preview.content	1
quarantine.preview.source	1
quarantine.preview.info	1
quarantine.preview.attachments	1
quarantine.preview.attachments.download	1
quarantine.preview.status	1
quarantine.reason.spam	1
quarantine.reason.virus	1
quarantine.reason.suspect	1
quarantine.reason.blacklisted	1
reports	1
reports.topvirustypes	1
reports.toprelays	1
reports.topspamrelays	1
reports.topvirusrelays	1
reports.topotherrelays	1
reports.topspamsenders	1
reports.topspamrecipients	1
reports.topvirustypes.export	1
reports.toprelays.export	1
reports.topspamrelays.export	1
reports.topvirusrelays.export	1
reports.topotherrelays.export	1
reports.topspamsenders.export	1
reports.topspamrecipients.export	1
reports.queuestatus	0
reports.queuestatus.export	0
quarantine.actions.forward.false_positive	1
quarantine.actions.forward.false_negative	1
quarantine.actions.forward.potential_virus	1
quarantine.actions.forward.support_submission	0
change_password	1
quarantine.actions.forward.custom	1
reports.messagecategories	1
reports.topvirussenders	1
reports.messagesblocked	1
reports.messagecategories.export	1
reports.messagesblocked.export	1
reports.topvirussenders.export	1
reports.categoriesbygroup	1
reports.categoriesbygroup.export	1
reports.policymarkhits	1
reports.policymarkhits.export	1
logsearch	1
logsearch.preview	1
logsearch.preview.rawlogs	1
reports.mtarejected	1
reports.mtarejected.export	1
logsearch.preview.rawlogs.message_log	1
reports.custommarkhits	1
\.

COPY cfg_profile (profile_id, profile_name, change_id) FROM stdin;
0	<default>	1
\.

COPY lock (capability, running) FROM stdin;
Centralized Digest	0
PRD Data Reductor	0
\.

COPY m_reason (reason_id, name) FROM stdin;
1	queue
2	spam
3	virus
\.

COPY pmx_forward_config (access_right, address, subject, description) FROM stdin;
quarantine.actions.forward.false_positive	not-spam@puremessage.net	PM-UNIX false positive report	Report false positive(s) to SophosLabs
quarantine.actions.forward.false_negative	is-spam@puremessage.net	PM-UNIX false negative report	Report false negative(s) to SophosLabs
quarantine.actions.forward.potential_virus	samples@sophos.com	PM-UNIX potential virus report	Report potential virus(es) to SophosLabs
quarantine.actions.forward.support_submission	puremessage-support@sophos.com	PM-UNIX sample submission to support	Report to Sophos Support
\.

COPY pmx_group (group_name, long_name) FROM stdin;
supergroup	Super user group
\.

COPY prd_applicable_dimension (dimension_name, fact_name, is_collected) FROM stdin;
period	msgs_processed_count	1
pmx_host	msgs_processed_count	1
sender	msgs_processed_count	1
relay	msgs_processed_count	1
probability_range	msgs_processed_count	1
msg_size_range	msgs_processed_count	1
direction	msgs_processed_count	1
spam_virus_rule	msgs_processed_count	1
quarantine_reason	msgs_processed_count	1
period	atchmt_processed_count	1
pmx_host	atchmt_processed_count	1
msg_size_range	atchmt_processed_count	1
atchmt_type	atchmt_processed_count	1
period	msgs_prcsd_per_rcpnt_count	1
pmx_host	msgs_prcsd_per_rcpnt_count	1
quarantine_reason	msgs_prcsd_per_rcpnt_count	1
recipient	msgs_prcsd_per_rcpnt_count	1
period	rule_hit_count	1
pmx_host	rule_hit_count	1
spam_virus_rule	rule_hit_count	1
period	user_action_count	1
pmx_host	user_action_count	1
recipient	user_action_count	1
user_action	user_action_count	1
relay	user_action_count	1
sender	user_action_count	1
probability_range	user_action_count	1
period	pmx_runtime_fact	1
fact_type	pmx_runtime_fact	1
pmx_host	pmx_runtime_fact	1
pmx_process	pmx_runtime_fact	1
concurrency_range	pmx_runtime_fact	1
rsrc_utlztn_range	pmx_runtime_fact	1
period	mta_action_count	1
pmx_host	mta_action_count	1
mta_action	mta_action_count	1
\.

COPY prd_dimension (name, is_purgeable, match_pattern, dont_match_pattern, increment, scale_type) FROM stdin;
recipient	1	\N	\N	\N	\N
sender	1	\N	\N	\N	\N
probability_range	0	\N	\N	\N	\N
msg_size_range	0	\N	\N	\N	\N
pmx_host	0	\N	\N	\N	\N
pmx_process	0	\N	\N	\N	\N
spam_virus_rule	1	\N	\N	\N	\N
period	1	\N	\N	\N	\N
fact_type	0	\N	\N	\N	\N
relay	1	\N	\N	\N	\N
rsrc_utlztn_range	0	\N	\N	\N	\N
concurrency_range	0	\N	\N	\N	\N
quarantine_reason	0	\N	\N	\N	\N
user_action	0	\N	\N	\N	\N
direction	0	\N	\N	\N	\N
atchmt_type	0	\N	\N	\N	\N
mta_action	0	\N	\N	\N	\N
policy_action	0	\N	\N	\N	\N
policy_reason	0	\N	\N	\N	\N
\.

COPY prd_direction (id, name) FROM stdin;
1	incoming
2	outgoing
\.

COPY prd_fact (name, is_collected) FROM stdin;
msgs_processed_count	1
atchmt_processed_count	1
msgs_prcsd_per_rcpnt_count	1
rule_hit_count	1
user_action_count	1
pmx_runtime_fact	1
policy_mark_count	1
mta_action_count	1
\.

COPY prd_msg_size_range (id, lo, hi) FROM stdin;
1	1	1
2	2	2
3	3	3
4	4	4
5	5	5
6	6	6
7	7	7
8	8	8
9	9	9
10	10	19
11	20	29
12	30	39
13	40	49
14	50	59
15	60	69
16	70	79
17	80	89
18	90	99
19	100	199
20	200	299
21	300	399
22	400	499
23	500	599
24	600	699
25	700	799
26	800	899
27	900	999
28	1000	1999
29	2000	2999
30	3000	3999
31	4000	4999
32	5000	5999
33	6000	6999
34	7000	7999
35	8000	8999
36	9000	9999
37	10000	19999
38	20000	29999
39	30000	39999
40	40000	49999
41	50000	59999
42	60000	69999
43	70000	79999
44	80000	89999
45	90000	99999
46	100000	199999
47	200000	299999
48	300000	399999
49	400000	499999
50	500000	599999
51	600000	699999
52	700000	799999
53	800000	899999
54	900000	999999
55	1000000	1999999
56	2000000	2999999
57	3000000	3999999
58	4000000	4999999
59	5000000	5999999
60	6000000	6999999
61	7000000	7999999
62	8000000	8999999
63	9000000	9999999
64	10000000	19999999
65	20000000	29999999
66	30000000	39999999
67	40000000	49999999
68	50000000	59999999
69	60000000	69999999
70	70000000	79999999
71	80000000	89999999
72	90000000	99999999
73	100000000	199999999
74	200000000	299999999
75	300000000	399999999
76	400000000	499999999
77	500000000	599999999
78	600000000	699999999
79	700000000	799999999
80	800000000	899999999
81	900000000	999999999
82	1000000000	1999999999
83	2000000000	2999999999
84	3000000000	3999999999
85	4000000000	4999999999
86	5000000000	5999999999
87	6000000000	6999999999
88	7000000000	7999999999
89	8000000000	8999999999
90	9000000000	9999999999
91	10000000000	19999999999
92	20000000000	29999999999
93	30000000000	39999999999
94	40000000000	49999999999
95	50000000000	59999999999
96	60000000000	69999999999
97	70000000000	79999999999
98	80000000000	89999999999
99	90000000000	99999999999
\.

COPY prd_mta_action (id, name) FROM stdin;
1	reject
2	tempfail
3	ok
\.

COPY prd_nullified_dimension (fact_name, action_name, dimension_name) FROM stdin;
msgs_processed_count	msg_proc_aggregation	relay
msgs_processed_count	msg_proc_aggregation	sender
\.

COPY prd_policy_action (id, name) FROM stdin;
1	reject
2	discard
3	ok
\.

COPY prd_probability_range (id, lo, hi) FROM stdin;
1	0	2
2	2	4
3	4	6
4	6	8
5	8	10
6	10	12
7	12	14
8	14	16
9	16	18
10	18	20
11	20	22
12	22	24
13	24	26
14	26	28
15	28	30
16	30	32
17	32	34
18	34	36
19	36	38
20	38	40
21	40	42
22	42	44
23	44	46
24	46	48
25	48	50
26	50	52
27	52	54
28	54	56
29	56	58
30	58	60
31	60	62
32	62	64
33	64	66
34	66	68
35	68	70
36	70	72
37	72	74
38	74	76
39	76	78
40	78	80
41	80	82
42	82	84
43	84	86
44	86	88
45	88	90
46	90	92
47	92	94
48	94	96
49	96	98
50	98	100
\.

COPY prd_quarantine_reason (id, name) FROM stdin;
1	spam
2	virus
3	other
\.

COPY prd_reduction_action (fact_name, action_name, action_type, age_threshold, recurrence, aggregation_period) FROM stdin;
rule_hit_count	rule_hit_count_expiry	expiry	172800	hourly	\N
policy_mark_count	policy_mark_count_expiry	expiry	172800	hourly	\N
pmx_runtime_fact	pmx_runtime_fact_aggregation	aggregation/avg	172800	hourly	3600
pmx_runtime_fact	user_action_count_expiry	expiry	63115200	daily	\N
user_action_count	user_action_count_aggregation	aggregation/sum	172800	hourly	3600
user_action_count	user_action_count_expiry	expiry	31557600	daily	\N
msgs_processed_count	msg_proc_expiry	expiry	63115200	daily	\N
msgs_processed_count	msg_proc_aggregation	aggregation/sum	129600	hourly	3600
msgs_prcsd_per_rcpnt_count	msg_proc_rcpnt_expiry	expiry	129600	hourly	\N
atchmt_processed_count	atchmt_processed_expiry	expiry	129600	hourly	\N
mta_action_count	mta_action_aggregation	aggregation/sum	172800	hourly	3600
mta_action_count	mta_action_expiry	expiry	63115200	daily	\N
\.

COPY prd_runtime_fact_type (id, name) FROM stdin;
1	disc_kbytes_written
2	disc_kbytes_read
3	disc_kbytes_allocated
4	ram_allocated
5	swap_allocated
6	cpu_utilized
7	load
8	uptime
9	total_quarantined_local
10	total_quarantined_central
11	new_quarantined_local
12	new_quarantined_central
13	quarantine_kbytes_local
14	quarantine_kbytes_central
15	disc_kbytes_free
\.

COPY prd_state (name, asof) FROM stdin;
rep-append	1969-12-31 16:00:00-08
rep-stable	1969-12-31 16:00:00-08
\.

COPY prd_user_action (id, name) FROM stdin;
2	approve
3	delete
\.

COPY read_write_permission (permission_name, permission_bit) FROM stdin;
read	2
write	4
\.

COPY read_write_permission_item (permission_item, default_permission_bits) FROM stdin;
configuration.policysettings	6
configuration.lists.allowed-senders-per-group	6
configuration.lists.allowed-relays-per-group	6
configuration.lists.blocked-senders-per-group	6
configuration.lists.blocked-relays-per-group	6
\.

ALTER TABLE ONLY group_multifile
    ADD CONSTRAINT group_multifile_multifile_id_key
            UNIQUE (multifile_id, group_name);

ALTER TABLE ONLY group_multifile
    ADD CONSTRAINT group_multifile_pkey
            PRIMARY KEY (group_multifile_id);

ALTER TABLE ONLY multifile
    ADD CONSTRAINT multifile_pkey
            PRIMARY KEY (multifile_id);

ALTER TABLE ONLY binary_permission_item
    ADD CONSTRAINT permission_item_unique
            UNIQUE (permission_item);

ALTER TABLE ONLY permission
    ADD CONSTRAINT permission_permission_bit_key
            UNIQUE (permission_bit);

ALTER TABLE ONLY cfg_attribute
    ADD CONSTRAINT pk_cfg_attribute
            PRIMARY KEY (resource_name, attribute_name);

ALTER TABLE ONLY cfg_binval
    ADD CONSTRAINT pk_cfg_binval
            PRIMARY KEY (instance_id);

ALTER TABLE ONLY cfg_instance
    ADD CONSTRAINT pk_cfg_instance
            PRIMARY KEY (instance_id);

ALTER TABLE ONLY cfg_kv
    ADD CONSTRAINT pk_cfg_kv
            PRIMARY KEY (instance_id, userkey, k_name);

ALTER TABLE ONLY cfg_location_instance
    ADD CONSTRAINT pk_cfg_location_instance
            PRIMARY KEY (location_name, instance_id);

ALTER TABLE ONLY cfg_profile
    ADD CONSTRAINT pk_cfg_profile
            PRIMARY KEY (profile_id);

ALTER TABLE ONLY cfg_resource
    ADD CONSTRAINT pk_cfg_resource
            PRIMARY KEY (resource_name);

ALTER TABLE ONLY group_admin
    ADD CONSTRAINT pk_group_admin
            PRIMARY KEY (group_admin_id);

ALTER TABLE ONLY group_admin_permission
    ADD CONSTRAINT pk_group_admin_permission
            PRIMARY KEY (group_admin_permission_id);

ALTER TABLE ONLY lock
    ADD CONSTRAINT pk_lock
            PRIMARY KEY (capability);

ALTER TABLE ONLY m_reason
    ADD CONSTRAINT pk_m_reason
            PRIMARY KEY (reason_id);

ALTER TABLE ONLY message
    ADD CONSTRAINT pk_message
            PRIMARY KEY (id);

ALTER TABLE ONLY message_body
    ADD CONSTRAINT pk_message_body
            PRIMARY KEY (message_id);

ALTER TABLE ONLY permission
    ADD CONSTRAINT pk_permission
            PRIMARY KEY (permission_name);

ALTER TABLE ONLY permission_item
    ADD CONSTRAINT pk_permission_item
            PRIMARY KEY (permission_item);

ALTER TABLE ONLY pmx_group
    ADD CONSTRAINT pk_pmx_group
            PRIMARY KEY (group_name);

ALTER TABLE ONLY pmx_user
    ADD CONSTRAINT pk_pmx_user
            PRIMARY KEY (user_name);

ALTER TABLE ONLY prd_applicable_dimension
    ADD CONSTRAINT pk_prd_applicable_dimension
            PRIMARY KEY (dimension_name, fact_name);

ALTER TABLE ONLY prd_concurrency_range
    ADD CONSTRAINT pk_prd_concurrency_range
            PRIMARY KEY (id);

ALTER TABLE ONLY prd_dimension
    ADD CONSTRAINT pk_prd_dimension
            PRIMARY KEY (name);

ALTER TABLE ONLY prd_fact
    ADD CONSTRAINT pk_prd_fact
            PRIMARY KEY (name);

ALTER TABLE ONLY prd_msg_size_range
    ADD CONSTRAINT pk_prd_msg_size_range
            PRIMARY KEY (id);

ALTER TABLE ONLY prd_mta_action
    ADD CONSTRAINT pk_prd_mta_action
            PRIMARY KEY (id);

ALTER TABLE ONLY prd_nullified_dimension
    ADD CONSTRAINT pk_prd_nullified_dimension
            PRIMARY KEY (fact_name, action_name, dimension_name);

ALTER TABLE ONLY prd_period
    ADD CONSTRAINT pk_prd_period
            PRIMARY KEY (id);

ALTER TABLE ONLY prd_pmx_host
    ADD CONSTRAINT pk_prd_pmx_host
            PRIMARY KEY (id);

ALTER TABLE ONLY prd_pmx_process
    ADD CONSTRAINT pk_prd_pmx_process
            PRIMARY KEY (id);

ALTER TABLE ONLY prd_policy_action
    ADD CONSTRAINT pk_prd_policy_action
            PRIMARY KEY (id);

ALTER TABLE ONLY prd_policy_mark
    ADD CONSTRAINT pk_prd_policy_mark
            PRIMARY KEY (id);

ALTER TABLE ONLY prd_policy_reason
    ADD CONSTRAINT pk_prd_policy_reason
            PRIMARY KEY (id);

ALTER TABLE ONLY prd_probability_range
    ADD CONSTRAINT pk_prd_probability_range
            PRIMARY KEY (id);

ALTER TABLE ONLY prd_quarantine_reason
    ADD CONSTRAINT pk_prd_quarantine_reason
            PRIMARY KEY (id);

ALTER TABLE ONLY prd_recipient
    ADD CONSTRAINT pk_prd_recipient
            PRIMARY KEY (id);

ALTER TABLE ONLY prd_reduction_action
    ADD CONSTRAINT pk_prd_reduction_action
            PRIMARY KEY (fact_name, action_name);

ALTER TABLE ONLY prd_relay
    ADD CONSTRAINT pk_prd_relay
            PRIMARY KEY (id);

ALTER TABLE ONLY prd_rsrc_utlztn_range
    ADD CONSTRAINT pk_prd_rsrc_utlztn_range
            PRIMARY KEY (id);

ALTER TABLE ONLY prd_runtime_fact_type
    ADD CONSTRAINT pk_prd_runtime_fact_type
            PRIMARY KEY (id);

ALTER TABLE ONLY prd_sender
    ADD CONSTRAINT pk_prd_sender
            PRIMARY KEY (id);

ALTER TABLE ONLY prd_spam_virus_rule
    ADD CONSTRAINT pk_prd_spam_virus_rule
            PRIMARY KEY (id);

ALTER TABLE ONLY prd_user_action
    ADD CONSTRAINT pk_prd_user_action
            PRIMARY KEY (id);

ALTER TABLE ONLY recipient
    ADD CONSTRAINT pk_recipient
            PRIMARY KEY (id);

ALTER TABLE ONLY session
    ADD CONSTRAINT pk_session
            PRIMARY KEY (id);

ALTER TABLE ONLY cfg_profile
    ADD CONSTRAINT unq_cfg_profile_name
            UNIQUE (profile_name);

ALTER TABLE ONLY digest_state
    ADD CONSTRAINT unq_dgst_state_usr_type
            UNIQUE (digest_user, digest_type);

ALTER TABLE ONLY group_admin
    ADD CONSTRAINT unq_group_admin_group_name_user_name
            UNIQUE (group_name, user_name);

ALTER TABLE ONLY group_admin_permission
    ADD CONSTRAINT unq_group_admin_permission_group_admin_id_permission_item
            UNIQUE (group_admin_id, permission_item);

ALTER TABLE ONLY m_reason
    ADD CONSTRAINT unq_m_reason_name
            UNIQUE (name);

ALTER TABLE ONLY message
    ADD CONSTRAINT unq_message_m_global_id
            UNIQUE (m_global_id);

ALTER TABLE ONLY message_recipient
    ADD CONSTRAINT unq_message_recipient
            UNIQUE (recipient_id, message_id);

ALTER TABLE ONLY pmdb_schema_event
    ADD CONSTRAINT unq_pmdb_schema_event
            UNIQUE (event_type, event_date);

ALTER TABLE ONLY prd_mta_action
    ADD CONSTRAINT unq_prd_mta_action
            UNIQUE (name);

ALTER TABLE ONLY prd_period
    ADD CONSTRAINT unq_prd_period_start_end
            UNIQUE (period_start, period_end);

ALTER TABLE ONLY prd_policy_action
    ADD CONSTRAINT unq_prd_policy_action
            UNIQUE (name);

ALTER TABLE ONLY prd_policy_mark
    ADD CONSTRAINT unq_prd_policy_mark
            UNIQUE (name);

ALTER TABLE ONLY prd_policy_reason
    ADD CONSTRAINT unq_prd_policy_reason
            UNIQUE (name);

ALTER TABLE ONLY prd_recipient
    ADD CONSTRAINT unq_prd_recipient_local_domain
            UNIQUE (local, domain);

ALTER TABLE ONLY prd_relay
    ADD CONSTRAINT unq_prd_relay
            UNIQUE (address);

ALTER TABLE ONLY prd_sender
    ADD CONSTRAINT unq_prd_sender_local_domain
            UNIQUE (local, domain);

ALTER TABLE ONLY prd_spam_virus_rule
    ADD CONSTRAINT unq_prd_spam_virus_rule
            UNIQUE (rule, type);

ALTER TABLE ONLY quarantine_rule
    ADD CONSTRAINT unq_quarantine_rule
            UNIQUE (rule_text, reason_id);

ALTER TABLE ONLY quarantine_rule_hit
    ADD CONSTRAINT unq_quarantine_rule_hit
            UNIQUE (message_id, quarantine_rule_id);

ALTER TABLE ONLY recipient
    ADD CONSTRAINT unq_recipient
            UNIQUE (local, domain);

CREATE INDEX group_admin_group_name_index
	    ON group_admin (group_name);

CREATE INDEX group_admin_user_name_index
	    ON group_admin (user_name);

CREATE UNIQUE INDEX group_policy_option_opt_in_index
	    ON group_policy_option_opt_in (name);

CREATE INDEX message_env_from_domain_index
	    ON message (env_from_domain);

CREATE INDEX message_env_from_local_index
	    ON message (env_from_local);

CREATE INDEX message_group_message_id_index
	    ON message_group (message_id);

CREATE INDEX message_h_from_domain_index
	    ON message (h_from_domain);

CREATE INDEX message_h_from_local_index
	    ON message (h_from_local);

CREATE INDEX message_h_message_id_index
	    ON message (h_message_id);

CREATE INDEX message_m_date_index
	    ON message (m_date);

CREATE INDEX message_m_queue_id_index
	    ON message (m_queue_id);

CREATE INDEX message_recipient_message_id_index
	    ON message_recipient (message_id);

CREATE INDEX prd_atchmt_processed_count_period_id_index
	    ON prd_atchmt_processed_count (period_id);

CREATE INDEX prd_connections_by_action_index
	    ON prd_connections_by_action (group_name, period_start, action);

CREATE INDEX prd_custom_mark_hits_index
	    ON prd_custom_mark_hits (group_name, period_start, mark_key);

CREATE INDEX prd_msgs_by_policy_block_index
	    ON prd_msgs_by_policy_block (group_name, period_start, blocked);

CREATE INDEX prd_msgs_by_type_index
	    ON prd_msgs_by_type (group_name, period_start, type, inbound);

CREATE INDEX prd_msgs_by_virus_index
	    ON prd_msgs_by_virus (group_name, period_start, virus_name, inbound);

CREATE INDEX prd_msgs_prcsd_per_rcpnt_count_period_id_index
	    ON prd_msgs_prcsd_per_rcpnt_count (period_id);

CREATE INDEX prd_msgs_prcsd_per_rcpnt_count_recipient_id_index
	    ON prd_msgs_prcsd_per_rcpnt_count (recipient_id);

CREATE INDEX prd_msgs_processed_count_period_id_pmx_host_id_quarantine_index
	    ON prd_msgs_processed_count (period_id, pmx_host_id, quarantine_reason_id);

CREATE INDEX prd_msgs_processed_count_relay_id_index
	    ON prd_msgs_processed_count (relay_id);

CREATE INDEX prd_msgs_processed_count_sender_id_index
	    ON prd_msgs_processed_count (sender_id);

CREATE INDEX prd_msgs_processed_count_spam_virus_rule_id_index
	    ON prd_msgs_processed_count (spam_virus_rule_id);

CREATE INDEX prd_mta_action_count_period_id_pmx_host_id_mta_action_id_index
	    ON prd_mta_action_count (period_id, pmx_host_id, mta_action_id);

CREATE INDEX prd_period_period_end_index
	    ON prd_period (period_end);

CREATE INDEX prd_pmx_runtime_fact_period_id_index
	    ON prd_pmx_runtime_fact (period_id);

CREATE INDEX prd_policy_mark_count_period_id_index
	    ON prd_policy_mark_count (period_id);

CREATE INDEX prd_recipient_domain_index
	    ON prd_recipient (domain);

CREATE INDEX prd_relays_by_type_index
	    ON prd_relays_by_type (group_name, period_start, type, relay, inbound);

CREATE INDEX prd_rule_hit_count_period_id_index
	    ON prd_rule_hit_count (period_id);

CREATE INDEX prd_rule_hit_count_spam_virus_rule_id_index
	    ON prd_rule_hit_count (spam_virus_rule_id);

CREATE INDEX prd_sender_domain_index
	    ON prd_sender (domain);

CREATE INDEX prd_senders_by_type_index
	    ON prd_senders_by_type (group_name, period_start, type, sender, inbound);

CREATE INDEX prd_spam_by_recipient_index
	    ON prd_spam_by_recipient (group_name, period_start, recipient);

CREATE UNIQUE INDEX prd_state_name_index
	    ON prd_state (name);

CREATE INDEX prd_user_action_count_period_id_index
	    ON prd_user_action_count (period_id);

CREATE INDEX prd_user_action_count_recipient_id_index
	    ON prd_user_action_count (recipient_id);

CREATE INDEX prd_user_action_count_relay_id_index
	    ON prd_user_action_count (relay_id);

CREATE INDEX prd_user_action_count_sender_id_index
	    ON prd_user_action_count (sender_id);

CREATE UNIQUE INDEX qrntn_rqst_idactn_index
	    ON qrntn_action_request (m_global_id, action)
	    WHERE ((local IS NULL) AND (domain IS NULL));

CREATE UNIQUE INDEX qrntn_rqst_idactnlcl_index
	    ON qrntn_action_request (m_global_id, action, local)
	    WHERE ((local IS NOT NULL) AND (domain IS NULL));

CREATE UNIQUE INDEX qrntn_rqst_idactnlcldmn_index
	    ON qrntn_action_request (m_global_id, action, local, domain)
	    WHERE ((local IS NOT NULL) AND (domain IS NOT NULL));

CREATE INDEX recipient_domain_index
	    ON recipient (domain);

CREATE RULE group_admin_permission_for_insert AS ON INSERT TO group_admin_permission DO NOTHING;

CREATE RULE permission_for_insert AS ON INSERT TO permission DO NOTHING;

CREATE RULE permission_item_for_insert AS ON INSERT TO permission_item DO NOTHING;

CREATE TRIGGER au_r_cfg_binval
    AFTER UPDATE ON cfg_binval
    FOR EACH ROW
    EXECUTE PROCEDURE upd_cfg_binval();

CREATE TRIGGER au_r_cfg_kv
    AFTER UPDATE ON cfg_kv
    FOR EACH ROW
    EXECUTE PROCEDURE upd_cfg_kv();

CREATE TRIGGER bd_r_cfg_binval
    BEFORE DELETE ON cfg_binval
    FOR EACH ROW
    EXECUTE PROCEDURE del_cfg_binval();

CREATE TRIGGER bd_r_cfg_kv
    BEFORE DELETE ON cfg_kv
    FOR EACH ROW
    EXECUTE PROCEDURE del_cfg_kv();

CREATE TRIGGER bd_r_cfg_listrow
    BEFORE DELETE ON cfg_listrow
    FOR EACH ROW
    EXECUTE PROCEDURE del_cfg_listrow();

CREATE TRIGGER bd_r_message_recipient
    BEFORE DELETE ON message_recipient
    FOR EACH ROW
    EXECUTE PROCEDURE delete_unmapped_rcpnts();

CREATE TRIGGER bi_r_cfg_binval
    BEFORE INSERT ON cfg_binval
    FOR EACH ROW
    EXECUTE PROCEDURE ins_cfg_binval();

CREATE TRIGGER bi_r_cfg_instance
    BEFORE INSERT ON cfg_instance
    FOR EACH ROW
    EXECUTE PROCEDURE ins_cfg_instance();

CREATE TRIGGER bi_r_cfg_kv
    BEFORE INSERT ON cfg_kv
    FOR EACH ROW
    EXECUTE PROCEDURE ins_cfg_kv();

CREATE TRIGGER bi_r_cfg_listrow
    BEFORE INSERT ON cfg_listrow
    FOR EACH ROW
    EXECUTE PROCEDURE ins_cfg_listrow();

CREATE TRIGGER bi_r_cfg_location
    BEFORE INSERT ON cfg_location
    FOR EACH ROW
    EXECUTE PROCEDURE ins_cfg_location();

ALTER TABLE ONLY group_multifile
    ADD CONSTRAINT "$1"
            FOREIGN KEY (multifile_id)
            REFERENCES multifile(multifile_id) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY group_multifile
    ADD CONSTRAINT "$2"
            FOREIGN KEY (group_name)
            REFERENCES pmx_group(group_name) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY cfg_attribute
    ADD CONSTRAINT fk_cfg_attr_res_name
            FOREIGN KEY (resource_name)
            REFERENCES cfg_resource(resource_name) ON DELETE CASCADE;

ALTER TABLE ONLY cfg_binval
    ADD CONSTRAINT fk_cfg_binval_cfg_instance
            FOREIGN KEY (instance_id)
            REFERENCES cfg_instance(instance_id) ON DELETE CASCADE;

ALTER TABLE ONLY cfg_instance
    ADD CONSTRAINT fk_cfg_inst_prof_id
            FOREIGN KEY (profile_id)
            REFERENCES cfg_profile(profile_id) ON DELETE CASCADE;

ALTER TABLE ONLY cfg_instance
    ADD CONSTRAINT fk_cfg_inst_res_name
            FOREIGN KEY (resource_name)
            REFERENCES cfg_resource(resource_name) ON DELETE CASCADE;

ALTER TABLE ONLY cfg_item_change
    ADD CONSTRAINT fk_cfg_item_change_cfg_instance
            FOREIGN KEY (instance_id)
            REFERENCES cfg_instance(instance_id) ON DELETE CASCADE;

ALTER TABLE ONLY cfg_kv
    ADD CONSTRAINT fk_cfg_kv_cfg_instance
            FOREIGN KEY (instance_id)
            REFERENCES cfg_instance(instance_id) ON DELETE CASCADE;

ALTER TABLE ONLY cfg_listrow
    ADD CONSTRAINT fk_cfg_listrow_cfg_instance
            FOREIGN KEY (instance_id)
            REFERENCES cfg_instance(instance_id) ON DELETE CASCADE;

ALTER TABLE ONLY group_admin
    ADD CONSTRAINT fk_group_admin_group_name
            FOREIGN KEY (group_name)
            REFERENCES pmx_group(group_name) ON DELETE CASCADE;

ALTER TABLE ONLY group_admin_permission
    ADD CONSTRAINT fk_group_admin_permission_group_admin_id
            FOREIGN KEY (group_admin_id)
            REFERENCES group_admin(group_admin_id) ON DELETE CASCADE;

ALTER TABLE ONLY group_admin_permission
    ADD CONSTRAINT fk_group_admin_permission_permission_item
            FOREIGN KEY (permission_item)
            REFERENCES permission_item(permission_item) ON DELETE CASCADE;

ALTER TABLE ONLY group_admin
    ADD CONSTRAINT fk_group_admin_user_name
            FOREIGN KEY (user_name)
            REFERENCES pmx_user(user_name) ON DELETE CASCADE;

ALTER TABLE ONLY message_group
    ADD CONSTRAINT fk_message_group_message_id
            FOREIGN KEY (message_id)
            REFERENCES message(id) ON DELETE CASCADE;

ALTER TABLE ONLY prd_applicable_dimension
    ADD CONSTRAINT fk_prd_app_dmnsn_dmnsn_name
            FOREIGN KEY (dimension_name)
            REFERENCES prd_dimension(name) ON DELETE CASCADE;

ALTER TABLE ONLY prd_applicable_dimension
    ADD CONSTRAINT fk_prd_app_dmnsn_fact_name
            FOREIGN KEY (fact_name)
            REFERENCES prd_fact(name) ON DELETE CASCADE;

ALTER TABLE ONLY prd_nullified_dimension
    ADD CONSTRAINT fk_prd_nlfd_dmnsn_dmnsn_name
            FOREIGN KEY (dimension_name)
            REFERENCES prd_dimension(name) ON DELETE CASCADE;

ALTER TABLE ONLY prd_reduction_action
    ADD CONSTRAINT fk_prd_rdctn_actn_fact_name
            FOREIGN KEY (fact_name)
            REFERENCES prd_fact(name) ON DELETE CASCADE;

ALTER TABLE ONLY message_body
    ADD CONSTRAINT rc_msg_body_msg_id
            FOREIGN KEY (message_id)
            REFERENCES message(id) ON DELETE CASCADE;

ALTER TABLE ONLY quarantine_rule_hit
    ADD CONSTRAINT rc_qrule_hit_msg_id
            FOREIGN KEY (message_id)
            REFERENCES message(id) ON DELETE CASCADE;

ALTER TABLE ONLY message_recipient
    ADD CONSTRAINT rc_rcpnt_map_msg_id
            FOREIGN KEY (message_id)
            REFERENCES message(id) ON DELETE CASCADE;

COMMENT ON SCHEMA public IS 'PMX 5.600';

